@@ -67,11 +67,11 @@ void lcCamera::Initialize()
 {
 	m_fovy = 30.0f;
 	m_zNear = 25.0f;
-	m_zFar = 12500.0f;
+	m_zFar = 50000.0f;
 
 	mState = 0;
 
-	m_pTR = NULL;
+	m_pTR = nullptr;
 	memset(m_strName, 0, sizeof(m_strName));
 }
 
@@ -155,11 +155,20 @@ bool lcCamera::ParseLDrawLine(QTextStream& Stream)
 		else if (Token == QLatin1String("ZFAR"))
 			Stream >> m_zFar;
 		else if (Token == QLatin1String("POSITION"))
+		{
 			Stream >> mPosition[0] >> mPosition[1] >> mPosition[2];
+			ChangeKey(mPositionKeys, mPosition, 1, true);
+		}
 		else if (Token == QLatin1String("TARGET_POSITION"))
+		{
 			Stream >> mTargetPosition[0] >> mTargetPosition[1] >> mTargetPosition[2];
+			ChangeKey(mTargetPositionKeys, mTargetPosition, 1, true);
+		}
 		else if (Token == QLatin1String("UP_VECTOR"))
+		{
 			Stream >> mUpVector[0] >> mUpVector[1] >> mUpVector[2];
+			ChangeKey(mUpVectorKeys, mUpVector, 1, true);
+		}
 		else if (Token == QLatin1String("POSITION_KEY"))
 			LoadKeysLDraw(Stream, mPositionKeys);
 		else if (Token == QLatin1String("TARGET_POSITION_KEY"))
@@ -464,11 +473,12 @@ void lcCamera::CopyPosition(const lcCamera* camera)
 	mPosition = camera->mPosition;
 	mTargetPosition = camera->mTargetPosition;
 	mUpVector = camera->mUpVector;
+	mState |= (camera->mState&LC_CAMERA_ORTHO);
 }
 
 void lcCamera::DrawInterface(lcContext* Context) const
 {
-	Context->SetProgram(LC_PROGRAM_SIMPLE);
+	Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 
 	lcMatrix44 ViewWorldMatrix = lcMatrix44AffineInverse(mWorldView);
 	ViewWorldMatrix.SetTranslation(lcVector3(0, 0, 0));
@@ -535,7 +545,7 @@ void lcCamera::DrawInterface(lcContext* Context) const
 	};
 
 	Context->SetVertexBufferPointer(Verts);
-	Context->SetVertexFormat(0, 3, 0, 0);
+	Context->SetVertexFormatPosition(3);
 	Context->SetIndexBufferPointer(Indices);
 
 	float LineWidth = lcGetPreferences().mLineWidth;
@@ -621,7 +631,7 @@ void lcCamera::RayTest(lcObjectRayTest& ObjectRayTest) const
 	lcVector3 End = lcMul31(ObjectRayTest.End, mWorldView);
 
 	float Distance;
-	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, NULL) && (Distance < ObjectRayTest.Distance))
+	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, nullptr) && (Distance < ObjectRayTest.Distance))
 	{
 		ObjectRayTest.ObjectSection.Object = const_cast<lcCamera*>(this);
 		ObjectRayTest.ObjectSection.Section = LC_CAMERA_SECTION_POSITION;
@@ -637,7 +647,7 @@ void lcCamera::RayTest(lcObjectRayTest& ObjectRayTest) const
 	Start = lcMul31(ObjectRayTest.Start, WorldView);
 	End = lcMul31(ObjectRayTest.End, WorldView);
 
-	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, NULL) && (Distance < ObjectRayTest.Distance))
+	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, nullptr) && (Distance < ObjectRayTest.Distance))
 	{
 		ObjectRayTest.ObjectSection.Object = const_cast<lcCamera*>(this);
 		ObjectRayTest.ObjectSection.Section = LC_CAMERA_SECTION_TARGET;
@@ -653,7 +663,7 @@ void lcCamera::RayTest(lcObjectRayTest& ObjectRayTest) const
 	Start = lcMul31(ObjectRayTest.Start, WorldView);
 	End = lcMul31(ObjectRayTest.End, WorldView);
 
-	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, NULL) && (Distance < ObjectRayTest.Distance))
+	if (lcBoundingBoxRayIntersectDistance(Min, Max, Start, End, &Distance, nullptr) && (Distance < ObjectRayTest.Distance))
 	{
 		ObjectRayTest.ObjectSection.Object = const_cast<lcCamera*>(this);
 		ObjectRayTest.ObjectSection.Section = LC_CAMERA_SECTION_UPVECTOR;
@@ -980,7 +990,7 @@ void lcCamera::StartTiledRendering(int tw, int th, int iw, int ih, float AspectR
 
 void lcCamera::GetTileInfo(int* row, int* col, int* width, int* height)
 {
-	if (m_pTR != NULL)
+	if (m_pTR != nullptr)
 	{
 		*row = m_pTR->mRows - m_pTR->mCurrentRow - 1;
 		*col = m_pTR->mCurrentColumn;
@@ -991,13 +1001,13 @@ void lcCamera::GetTileInfo(int* row, int* col, int* width, int* height)
 
 bool lcCamera::EndTile()
 {
-	if (m_pTR != NULL)
+	if (m_pTR != nullptr)
 	{
 		if (m_pTR->EndTile())
 			return true;
 
 		delete m_pTR;
-		m_pTR = NULL;
+		m_pTR = nullptr;
 	}
 
 	return false;
