@@ -1,6 +1,5 @@
 #include "lc_global.h"
 #include "lc_commands.h"
-#include "system.h"
 
 lcCommand gCommands[LC_NUM_COMMANDS] =
 {
@@ -46,6 +45,13 @@ lcCommand gCommands[LC_NUM_COMMANDS] =
 		QT_TRANSLATE_NOOP("Status", "Save a picture of the current view"),
 		QT_TRANSLATE_NOOP("Shortcut", "")
 	},
+	// LC_FILE_IMPORT_LDD
+	{
+		QT_TRANSLATE_NOOP("Action", "File.Import.LDD"),
+		QT_TRANSLATE_NOOP("Menu", "&LEGO Digital Designer..."),
+		QT_TRANSLATE_NOOP("Status", "Import a file in LEGO Digital Designer LXF format"),
+		QT_TRANSLATE_NOOP("Shortcut", "")
+	},
 	// LC_FILE_EXPORT_3DS
 	{
 		QT_TRANSLATE_NOOP("Action", "File.Export.3DS"),
@@ -228,6 +234,13 @@ lcCommand gCommands[LC_NUM_COMMANDS] =
 		QT_TRANSLATE_NOOP("Status", "Select objects by name"),
 		QT_TRANSLATE_NOOP("Shortcut", "")
 	},
+	// LC_EDIT_SELECT_BY_COLOR
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.SelectByColor"),
+		QT_TRANSLATE_NOOP("Menu", "Select by Col&or..."),
+		QT_TRANSLATE_NOOP("Status", "Select pieces by color"),
+		QT_TRANSLATE_NOOP("Shortcut", "")
+	},
 	// LC_EDIT_LOCK_X
 	{
 		QT_TRANSLATE_NOOP("Action", "Edit.Lock.LockX"),
@@ -949,6 +962,13 @@ lcCommand gCommands[LC_NUM_COMMANDS] =
 		QT_TRANSLATE_NOOP("Status", "Delete selected objects"),
 		QT_TRANSLATE_NOOP("Shortcut", "Delete")
 	},
+	// LC_PIECE_DUPLICATE
+	{
+		QT_TRANSLATE_NOOP("Action", "Piece.Duplicate"),
+		QT_TRANSLATE_NOOP("Menu", "&Duplicate"),
+		QT_TRANSLATE_NOOP("Status", "Create a copy of the selected pieces"),
+		QT_TRANSLATE_NOOP("Shortcut", "Ctrl+D")
+	},
 	// LC_PIECE_RESET_PIVOT_POINT
 	{
 		QT_TRANSLATE_NOOP("Action", "Piece.ResetPivotPoint"),
@@ -1420,7 +1440,7 @@ lcCommand gCommands[LC_NUM_COMMANDS] =
 	}
 };
 
-LC_CASSERT(sizeof(gCommands)/sizeof(gCommands[0]) == LC_NUM_COMMANDS);
+static_assert(sizeof(gCommands)/sizeof(gCommands[0]) == LC_NUM_COMMANDS, "Array size mismatch.");
 
 const char* gToolNames[LC_NUM_TOOLS] =
 {
@@ -1440,4 +1460,4 @@ const char* gToolNames[LC_NUM_TOOLS] =
 	QT_TRANSLATE_NOOP("Mouse", "ZoomRegion")     // LC_TOOL_ZOOM_REGION
 };
 
-LC_CASSERT(sizeof(gToolNames) / sizeof(gToolNames[0]) == LC_NUM_TOOLS);
+static_assert(sizeof(gToolNames) / sizeof(gToolNames[0]) == LC_NUM_TOOLS, "Array size mismatch.");
