@@ -9,6 +9,7 @@ enum lcCommandId
 	LC_FILE_SAVE,
 	LC_FILE_SAVEAS,
 	LC_FILE_SAVE_IMAGE,
+	LC_FILE_IMPORT_LDD,
 	LC_FILE_EXPORT_3DS,
 	LC_FILE_EXPORT_HTML,
 	LC_FILE_EXPORT_BRICKLINK,
@@ -37,6 +38,7 @@ enum lcCommandId
 	LC_EDIT_SELECT_NONE,
 	LC_EDIT_SELECT_INVERT,
 	LC_EDIT_SELECT_BY_NAME,
+	LC_EDIT_SELECT_BY_COLOR,
 	LC_EDIT_LOCK_X,
 	LC_EDIT_LOCK_Y,
 	LC_EDIT_LOCK_Z,
@@ -146,6 +148,7 @@ enum lcCommandId
 	LC_VIEW_PROJECTION_LAST = LC_VIEW_PROJECTION_ORTHO,
 	LC_PIECE_INSERT,
 	LC_PIECE_DELETE,
+	LC_PIECE_DUPLICATE,
 	LC_PIECE_RESET_PIVOT_POINT,
 	LC_PIECE_CONTROL_POINT_INSERT,
 	LC_PIECE_CONTROL_POINT_REMOVE,
