@@ -18,70 +18,25 @@
 #define GL_STATIC_DRAW_ARB GL_STATIC_DRAW
 #endif
 
-lcProgram lcContext::mPrograms[LC_NUM_PROGRAMS];
-
-static int lcOpaqueRenderMeshCompare(const void* Elem1, const void* Elem2)
-{
-	lcRenderMesh* Mesh1 = (lcRenderMesh*)Elem1;
-	lcRenderMesh* Mesh2 = (lcRenderMesh*)Elem2;
-
-	if (Mesh1->Mesh < Mesh2->Mesh)
-		return -1;
-
-	if (Mesh1->Mesh > Mesh2->Mesh)
-		return 1;
-
-	return 0;
-}
-
-static int lcTranslucentRenderMeshCompare(const void* Elem1, const void* Elem2)
-{
-	lcRenderMesh* Mesh1 = (lcRenderMesh*)Elem1;
-	lcRenderMesh* Mesh2 = (lcRenderMesh*)Elem2;
-
-	if (Mesh1->Distance < Mesh2->Distance)
-		return 1;
-
-	if (Mesh1->Distance > Mesh2->Distance)
-		return -1;
-
-	return 0;
-}
-
-lcScene::lcScene()
-	: mOpaqueMeshes(0, 1024), mTranslucentMeshes(0, 1024), mInterfaceObjects(0, 1024)
-{
-}
-
-void lcScene::Begin(const lcMatrix44& ViewMatrix)
-{
-	mViewMatrix = ViewMatrix;
-	mOpaqueMeshes.RemoveAll();
-	mTranslucentMeshes.RemoveAll();
-	mInterfaceObjects.RemoveAll();
-}
-
-void lcScene::End()
-{
-	qsort(&mOpaqueMeshes[0], mOpaqueMeshes.GetSize(), sizeof(mOpaqueMeshes[0]), lcOpaqueRenderMeshCompare);
-	qsort(&mTranslucentMeshes[0], mTranslucentMeshes.GetSize(), sizeof(mTranslucentMeshes[0]), lcTranslucentRenderMeshCompare);
-}
+lcProgram lcContext::mPrograms[LC_NUM_MATERIALS];
 
 lcContext::lcContext()
 {
 	mVertexBufferObject = 0;
 	mIndexBufferObject = 0;
-	mVertexBufferPointer = NULL;
-	mIndexBufferPointer = NULL;
+	mVertexBufferPointer = nullptr;
+	mIndexBufferPointer = nullptr;
 	mVertexBufferOffset = (char*)~0;
 
+	mNormalEnabled = false;
 	mTexCoordEnabled = false;
 	mColorEnabled = false;
 
-	mTexture = NULL;
+	mTexture = 0;
 	mLineWidth = 1.0f;
 #ifndef LC_OPENGLES
 	mMatrixMode = GL_MODELVIEW;
+	mTextureEnabled = false;
 #endif
 
 	mFramebufferObject = 0;
@@ -99,7 +54,7 @@ lcContext::lcContext()
 	mProjectionMatrixDirty = false;
 	mViewProjectionMatrixDirty = false;
 
-	mProgramType = LC_NUM_PROGRAMS;
+	mMaterialType = LC_NUM_MATERIALS;
 }
 
 lcContext::~lcContext()
@@ -121,10 +76,19 @@ void lcContext::CreateShaderPrograms()
 #define LC_PIXEL_INPUT "in mediump "
 #define LC_PIXEL_OUTPUT "#define gl_FragColor FragColor\nout mediump vec4 gl_FragColor;\n"
 #endif
-	
-	const char* VertexShaders[LC_NUM_PROGRAMS] =
+#define LC_PIXEL_FAKE_LIGHTING \
+	"	vec3 Normal = normalize(PixelNormal);\n" \
+	"	vec3 LightDirection = normalize(PixelPosition - LightPosition);" \
+	"	vec3 VertexToEye = normalize(EyePosition - PixelPosition);\n" \
+	"	vec3 LightReflect = normalize(reflect(-LightDirection, Normal));\n" \
+	"	float Specular = abs(dot(VertexToEye, LightReflect));\n" \
+	"	Specular = min(pow(Specular, 8.0), 1.0) * 0.25;\n" \
+	"	vec3 SpecularColor = vec3(Specular, Specular, Specular);\n" \
+	"	float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
+
+	const char* VertexShaders[LC_NUM_MATERIALS] =
 	{
-		// LC_PROGRAM_SIMPLE
+		// LC_MATERIAL_UNLIT_COLOR
 		LC_SHADER_VERSION
 		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
 		"uniform mat4 WorldViewProjectionMatrix;\n"
@@ -132,7 +96,7 @@ void lcContext::CreateShaderPrograms()
 		"{\n"
 		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
 		"}\n",
-		// LC_PROGRAM_TEXTURE
+		// LC_MATERIAL_UNLIT_TEXTURE_MODULATE
 		LC_SHADER_VERSION
 		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
 		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
@@ -143,7 +107,18 @@ void lcContext::CreateShaderPrograms()
 		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
 		"	PixelTexCoord = VertexTexCoord;\n"
 		"}\n",
-		// LC_PROGRAM_VERTEX_COLOR
+		// LC_MATERIAL_UNLIT_TEXTURE_DECAL
+		LC_SHADER_VERSION
+		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
+		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
+		LC_VERTEX_OUTPUT "vec2 PixelTexCoord;\n"
+		"uniform mat4 WorldViewProjectionMatrix;\n"
+		"void main()\n"
+		"{\n"
+		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
+		"	PixelTexCoord = VertexTexCoord;\n"
+		"}\n",
+		// LC_MATERIAL_UNLIT_VERTEX_COLOR
 		LC_SHADER_VERSION
 		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
 		LC_VERTEX_INPUT "vec4 VertexColor;\n"
@@ -153,43 +128,117 @@ void lcContext::CreateShaderPrograms()
 		"{\n"
 		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
 		"	PixelColor = VertexColor;\n"
+		"}\n",
+		// LC_MATERIAL_FAKELIT_COLOR
+		LC_SHADER_VERSION
+		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
+		LC_VERTEX_INPUT "vec3 VertexNormal;\n"
+		LC_VERTEX_OUTPUT "vec3 PixelPosition;\n"
+		LC_VERTEX_OUTPUT "vec3 PixelNormal;\n"
+		"uniform mat4 WorldViewProjectionMatrix;\n"
+		"uniform mat4 WorldMatrix;\n"
+		"void main()\n"
+		"{\n"
+		"	PixelPosition = (WorldMatrix * vec4(VertexPosition, 1.0)).xyz;\n"
+		"   PixelNormal = (WorldMatrix * vec4(VertexNormal, 0.0)).xyz;\n"
+		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
+		"}\n",
+		// LC_MATERIAL_FAKELIT_TEXTURE_DECAL
+		LC_SHADER_VERSION
+		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
+		LC_VERTEX_INPUT "vec3 VertexNormal;\n"
+		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
+		LC_VERTEX_OUTPUT "vec3 PixelPosition;\n"
+		LC_VERTEX_OUTPUT "vec3 PixelNormal;\n"
+		LC_VERTEX_OUTPUT "vec2 PixelTexCoord;\n"
+		"uniform mat4 WorldViewProjectionMatrix;\n"
+		"uniform mat4 WorldMatrix;\n"
+		"void main()\n"
+		"{\n"
+		"	PixelPosition = (WorldMatrix * vec4(VertexPosition, 1.0)).xyz;\n"
+		"   PixelNormal = (WorldMatrix * vec4(VertexNormal, 0.0)).xyz;\n"
+		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
+		"	PixelTexCoord = VertexTexCoord;\n"
 		"}\n"
 	};
 
-	const char* FragmentShaders[LC_NUM_PROGRAMS] =
+	const char* FragmentShaders[LC_NUM_MATERIALS] =
 	{
-		// LC_PROGRAM_SIMPLE
+		// LC_MATERIAL_UNLIT_COLOR
 		LC_SHADER_VERSION
 		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 Color;\n"
+		"uniform mediump vec4 MaterialColor;\n"
 		"void main()\n"
 		"{\n"
-		"	gl_FragColor = Color;\n"
+		"	gl_FragColor = MaterialColor;\n"
 		"}\n",
-		// LC_PROGRAM_TEXTURE
+		// LC_MATERIAL_UNLIT_TEXTURE_MODULATE
 		LC_SHADER_VERSION
 		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
 		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 Color;\n"
+		"uniform mediump vec4 MaterialColor;\n"
 		"uniform sampler2D Texture;\n"
 		"void main()\n"
 		"{\n"
-		"	gl_FragColor = texture2D(Texture, PixelTexCoord) * Color;\n"
+		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		"	gl_FragColor = vec4(MaterialColor.rgb, TexelColor.a * MaterialColor.a);\n"
 		"}\n",
-		// LC_PROGRAM_VERTEX_COLOR
+		// LC_MATERIAL_UNLIT_TEXTURE_DECAL
+		LC_SHADER_VERSION
+		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
+		LC_PIXEL_OUTPUT
+		"uniform mediump vec4 MaterialColor;\n"
+		"uniform sampler2D Texture;\n"
+		"void main()\n"
+		"{\n"
+		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		"	gl_FragColor = vec4(mix(MaterialColor.xyz, TexelColor.xyz, TexelColor.a), MaterialColor.a);\n"
+		"}\n",
+		// LC_MATERIAL_UNLIT_VERTEX_COLOR
 		LC_SHADER_VERSION
 		LC_PIXEL_INPUT "vec4 PixelColor;\n"
 		LC_PIXEL_OUTPUT
 		"void main()\n"
 		"{\n"
 		"	gl_FragColor = PixelColor;\n"
+		"}\n",
+		// LC_MATERIAL_FAKELIT_COLOR
+		LC_SHADER_VERSION
+		LC_PIXEL_INPUT "vec3 PixelPosition;\n"
+		LC_PIXEL_INPUT "vec3 PixelNormal;\n"
+		LC_PIXEL_OUTPUT
+		"uniform mediump vec4 MaterialColor;\n"
+		"uniform mediump vec3 LightPosition;\n"
+		"uniform mediump vec3 EyePosition;\n"
+		"void main()\n"
+		"{\n"
+		LC_PIXEL_FAKE_LIGHTING
+		"	vec3 DiffuseColor = MaterialColor.rgb * Diffuse;\n"
+		"	gl_FragColor = vec4(DiffuseColor + SpecularColor, MaterialColor.a);\n"
+		"}\n",
+		// LC_MATERIAL_FAKELIT_TEXTURE_DECAL
+		LC_SHADER_VERSION
+		LC_PIXEL_INPUT "vec3 PixelPosition;\n"
+		LC_PIXEL_INPUT "vec3 PixelNormal;\n"
+		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
+		LC_PIXEL_OUTPUT
+		"uniform mediump vec4 MaterialColor;\n"
+		"uniform mediump vec3 LightPosition;\n"
+		"uniform mediump vec3 EyePosition;\n"
+		"uniform sampler2D Texture;\n"
+		"void main()\n"
+		"{\n"
+		LC_PIXEL_FAKE_LIGHTING
+		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		"	vec3 DiffuseColor = mix(MaterialColor.xyz, TexelColor.xyz, TexelColor.a) * Diffuse;\n"
+		"	gl_FragColor = vec4(DiffuseColor + SpecularColor, MaterialColor.a);\n"
 		"}\n"
 	};
 
-	for (int ProgramIdx = 0; ProgramIdx < LC_NUM_PROGRAMS; ProgramIdx++)
+	for (int MaterialType = 0; MaterialType < LC_NUM_MATERIALS; MaterialType++)
 	{
 		GLuint VertexShader = glCreateShader(GL_VERTEX_SHADER);
-		glShaderSource(VertexShader, 1, &VertexShaders[ProgramIdx], NULL);
+		glShaderSource(VertexShader, 1, &VertexShaders[MaterialType], nullptr);
 		glCompileShader(VertexShader);
 
 #ifndef QT_NO_DEBUG
@@ -210,7 +259,7 @@ void lcContext::CreateShaderPrograms()
 #endif
 
 		GLuint FragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
-		glShaderSource(FragmentShader, 1, &FragmentShaders[ProgramIdx], NULL);
+		glShaderSource(FragmentShader, 1, &FragmentShaders[MaterialType], nullptr);
 		glCompileShader(FragmentShader);
 
 #ifndef QT_NO_DEBUG
@@ -236,6 +285,7 @@ void lcContext::CreateShaderPrograms()
 		glAttachShader(Program, FragmentShader);
 
 		glBindAttribLocation(Program, LC_ATTRIB_POSITION, "VertexPosition");
+		glBindAttribLocation(Program, LC_ATTRIB_NORMAL, "VertexNormal");
 		glBindAttribLocation(Program, LC_ATTRIB_TEXCOORD, "VertexTexCoord");
 		glBindAttribLocation(Program, LC_ATTRIB_COLOR, "VertexColor");
 
@@ -256,14 +306,17 @@ void lcContext::CreateShaderPrograms()
 			QByteArray InfoLog;
 			InfoLog.resize(Length);
 			glGetProgramInfoLog(Program, Length, &Length, InfoLog.data());
- 
+
 			glDeleteProgram(Program);
 			Program = 0;
 		}
 
-		mPrograms[ProgramIdx].Object = Program;
-		mPrograms[ProgramIdx].MatrixLocation = glGetUniformLocation(Program, "WorldViewProjectionMatrix");
-		mPrograms[ProgramIdx].ColorLocation = glGetUniformLocation(Program, "Color");
+		mPrograms[MaterialType].Object = Program;
+		mPrograms[MaterialType].WorldViewProjectionMatrixLocation = glGetUniformLocation(Program, "WorldViewProjectionMatrix");
+		mPrograms[MaterialType].WorldMatrixLocation = glGetUniformLocation(Program, "WorldMatrix");
+		mPrograms[MaterialType].MaterialColorLocation = glGetUniformLocation(Program, "MaterialColor");
+		mPrograms[MaterialType].LightPositionLocation = glGetUniformLocation(Program, "LightPosition");
+		mPrograms[MaterialType].EyePositionLocation = glGetUniformLocation(Program, "EyePosition");
 	}
 }
 
@@ -280,8 +333,11 @@ void lcContext::DestroyResources()
 	if (!gSupportsShaderObjects)
 		return;
 
-	for (int ProgramIdx = 0; ProgramIdx < LC_NUM_PROGRAMS; ProgramIdx++)
-		glDeleteProgram(mPrograms[ProgramIdx].Object);
+	for (int MaterialType = 0; MaterialType < LC_NUM_MATERIALS; MaterialType++)
+	{
+		glDeleteProgram(mPrograms[MaterialType].Object);
+		mPrograms[MaterialType].Object = 0;
+	}
 }
 
 void lcContext::SetDefaultState()
@@ -289,6 +345,8 @@ void lcContext::SetDefaultState()
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
 
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
 	glEnable(GL_POLYGON_OFFSET_FILL);
 	glPolygonOffset(0.5f, 0.1f);
 
@@ -301,31 +359,35 @@ void lcContext::SetDefaultState()
 	if (gSupportsShaderObjects)
 	{
 		glEnableVertexAttribArray(LC_ATTRIB_POSITION);
+		glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
 		glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
 		glDisableVertexAttribArray(LC_ATTRIB_COLOR);
 	}
 	else
 	{
 		glEnableClientState(GL_VERTEX_ARRAY);
+		glDisableClientState(GL_NORMAL_ARRAY);
 		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 		glDisableClientState(GL_COLOR_ARRAY);
 
-		glVertexPointer(3, GL_FLOAT, 0, NULL);
-		glTexCoordPointer(2, GL_FLOAT, 0, NULL);
-		glColorPointer(4, GL_FLOAT, 0, NULL);
+		glVertexPointer(3, GL_FLOAT, 0, nullptr);
+		glNormalPointer(GL_BYTE, 0, nullptr);
+		glTexCoordPointer(2, GL_FLOAT, 0, nullptr);
+		glColorPointer(4, GL_FLOAT, 0, nullptr);
 	}
 
+	mNormalEnabled = false;
 	mTexCoordEnabled = false;
 	mColorEnabled = false;
 
 	mVertexBufferObject = 0;
 	mIndexBufferObject = 0;
-	mVertexBufferPointer = NULL;
-	mIndexBufferPointer = NULL;
+	mVertexBufferPointer = nullptr;
+	mIndexBufferPointer = nullptr;
 	mVertexBufferOffset = (char*)~0;
 
-	glDisable(GL_TEXTURE_2D);
-	mTexture = NULL;
+	glBindTexture(GL_TEXTURE_2D, 0);
+	mTexture = 0;
 
 	glLineWidth(1.0f);
 	mLineWidth = 1.0f;
@@ -333,26 +395,83 @@ void lcContext::SetDefaultState()
 	if (gSupportsShaderObjects)
 	{
 		glUseProgram(0);
-		mProgramType = LC_NUM_PROGRAMS;
+		mMaterialType = LC_NUM_MATERIALS;
 	}
 	else
 	{
 #ifndef LC_OPENGLES
 		glMatrixMode(GL_MODELVIEW);
 		mMatrixMode = GL_MODELVIEW;
+		glShadeModel(GL_FLAT);
+
+		glDisable(GL_TEXTURE_2D);
+		mTextureEnabled = false;
 #endif
 	}
 }
 
-void lcContext::SetProgram(lcProgramType ProgramType)
+void lcContext::ClearResources()
 {
-	if (!gSupportsShaderObjects || mProgramType == ProgramType)
+	ClearVertexBuffer();
+	ClearIndexBuffer();
+	BindTexture(0);
+}
+
+void lcContext::SetMaterial(lcMaterialType MaterialType)
+{
+	if (MaterialType == mMaterialType)
 		return;
 
-	glUseProgram(mPrograms[ProgramType].Object);
-	mProgramType = ProgramType;
-	mColorDirty = true;
-	mWorldMatrixDirty = true;
+	mMaterialType = MaterialType;
+
+	if (gSupportsShaderObjects)
+	{
+		glUseProgram(mPrograms[MaterialType].Object);
+		mColorDirty = true;
+		mWorldMatrixDirty = true; // todo: change dirty to a bitfield and set the lighting constants dirty here
+		mViewMatrixDirty = true;
+	}
+	else
+	{
+#ifndef LC_OPENGLES
+		switch (MaterialType)
+		{
+		case LC_MATERIAL_UNLIT_TEXTURE_MODULATE:
+			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+
+			if (!mTextureEnabled)
+			{
+				glEnable(GL_TEXTURE_2D);
+				mTextureEnabled = true;
+			}
+			break;
+
+		case LC_MATERIAL_FAKELIT_TEXTURE_DECAL:
+		case LC_MATERIAL_UNLIT_TEXTURE_DECAL:
+			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
+
+			if (!mTextureEnabled)
+			{
+				glEnable(GL_TEXTURE_2D);
+				mTextureEnabled = true;
+			}
+			break;
+
+		case LC_MATERIAL_UNLIT_COLOR:
+		case LC_MATERIAL_UNLIT_VERTEX_COLOR:
+		case LC_MATERIAL_FAKELIT_COLOR:
+			if (mTextureEnabled)
+			{
+				glDisable(GL_TEXTURE_2D);
+				mTextureEnabled = false;
+			}
+			break;
+
+		case LC_NUM_MATERIALS:
+			break;
+		}
+#endif
+	}
 }
 
 void lcContext::SetViewport(int x, int y, int Width, int Height)
@@ -369,17 +488,23 @@ void lcContext::SetLineWidth(float LineWidth)
 	mLineWidth = LineWidth;
 }
 
-void lcContext::SetTextureMode(lcTextureMode TextureMode)
+void lcContext::SetSmoothShading(bool Smooth)
 {
 #ifndef LC_OPENGLES
-	if (!gSupportsShaderObjects)
-	{
-		const GLenum ModeTable[] = { GL_DECAL, GL_REPLACE, GL_MODULATE };
-		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, ModeTable[TextureMode]);
-	}
+	if (gSupportsShaderObjects)
+		glShadeModel(Smooth ? GL_SMOOTH : GL_FLAT);
 #endif
 }
 
+void lcContext::BindTexture(GLuint Texture)
+{
+	if (mTexture == Texture)
+		return;
+
+	glBindTexture(GL_TEXTURE_2D, Texture);
+	mTexture = Texture;
+}
+
 void lcContext::SetColor(float Red, float Green, float Blue, float Alpha)
 {
 	SetColor(lcVector4(Red, Green, Blue, Alpha));
@@ -415,16 +540,16 @@ bool lcContext::BeginRenderToTexture(int Width, int Height)
 
 		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, mFramebufferObject);
 
-		glBindTexture(GL_TEXTURE_2D, mFramebufferTexture);
+		BindTexture(mFramebufferTexture);
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
 		glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, mFramebufferTexture, 0);
 
 		glBindRenderbuffer(GL_RENDERBUFFER, mDepthRenderbufferObject);
 		glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT24, Width, Height);
 		glFramebufferRenderbuffer(GL_DRAW_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, mDepthRenderbufferObject);
 
-		glBindTexture(GL_TEXTURE_2D, 0);
+		BindTexture(0);
 		glBindFramebuffer(GL_FRAMEBUFFER, mFramebufferObject);
 
 		if (glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
@@ -442,9 +567,9 @@ bool lcContext::BeginRenderToTexture(int Width, int Height)
 		glGenFramebuffersEXT(1, &mFramebufferObject); 
 		glGenTextures(1, &mFramebufferTexture); 
 
-		glBindTexture(GL_TEXTURE_2D, mFramebufferTexture); 
+		BindTexture(mFramebufferTexture); 
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL); 
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
 
 		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject); 
 		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, mFramebufferTexture, 0); 
@@ -457,7 +582,7 @@ bool lcContext::BeginRenderToTexture(int Width, int Height)
 
 		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, mDepthRenderbufferObject); 
 
-		glBindTexture(GL_TEXTURE_2D, 0);
+		BindTexture(0);
 		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject);
 
 		if (glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) != GL_FRAMEBUFFER_COMPLETE_EXT)
@@ -567,7 +692,8 @@ lcVertexBuffer lcContext::CreateVertexBuffer(int Size, const void* Data)
 	else
 	{
 		VertexBuffer.Pointer = malloc(Size);
-		memcpy(VertexBuffer.Pointer, Data, Size);
+		if (VertexBuffer.Pointer)
+			memcpy(VertexBuffer.Pointer, Data, Size);
 	}
 
 	return VertexBuffer;
@@ -593,7 +719,7 @@ void lcContext::DestroyVertexBuffer(lcVertexBuffer& VertexBuffer)
 		free(VertexBuffer.Pointer);
 	}
 
-	VertexBuffer.Pointer = NULL;
+	VertexBuffer.Pointer = nullptr;
 }
 
 lcIndexBuffer lcContext::CreateIndexBuffer(int Size, const void* Data)
@@ -612,7 +738,8 @@ lcIndexBuffer lcContext::CreateIndexBuffer(int Size, const void* Data)
 	else
 	{
 		IndexBuffer.Pointer = malloc(Size);
-		memcpy(IndexBuffer.Pointer, Data, Size);
+		if (IndexBuffer.Pointer)
+			memcpy(IndexBuffer.Pointer, Data, Size);
 	}
 
 	return IndexBuffer;
@@ -638,12 +765,12 @@ void lcContext::DestroyIndexBuffer(lcIndexBuffer& IndexBuffer)
 		free(IndexBuffer.Pointer);
 	}
 
-	IndexBuffer.Pointer = NULL;
+	IndexBuffer.Pointer = nullptr;
 }
 
 void lcContext::ClearVertexBuffer()
 {
-	mVertexBufferPointer = NULL;
+	mVertexBufferPointer = nullptr;
 	mVertexBufferOffset = (char*)~0;
 
 	if (mVertexBufferObject)
@@ -652,11 +779,38 @@ void lcContext::ClearVertexBuffer()
 		mVertexBufferObject = 0;
 	}
 
-	if (mTexCoordEnabled)
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+	if (gSupportsShaderObjects)
+	{
+		if (mNormalEnabled)
+			glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
 
-	if (mColorEnabled)
-		glDisableClientState(GL_COLOR_ARRAY);
+		if (mTexCoordEnabled)
+			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
+
+		if (mColorEnabled)
+			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
+
+		glVertexAttribPointer(LC_ATTRIB_POSITION, 3, GL_FLOAT, false, 0, nullptr);
+		glVertexAttribPointer(LC_ATTRIB_NORMAL, 4, GL_FLOAT, false, 0, nullptr);
+		glVertexAttribPointer(LC_ATTRIB_TEXCOORD, 2, GL_FLOAT, false, 0, nullptr);
+		glVertexAttribPointer(LC_ATTRIB_COLOR, 4, GL_FLOAT, false, 0, nullptr);
+	}
+	else
+	{
+		if (mNormalEnabled)
+			glDisableClientState(GL_NORMAL_ARRAY);
+
+		if (mTexCoordEnabled)
+			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+		if (mColorEnabled)
+			glDisableClientState(GL_COLOR_ARRAY);
+
+		glVertexPointer(3, GL_FLOAT, 0, nullptr);
+		glNormalPointer(GL_BYTE, 0, nullptr);
+		glTexCoordPointer(2, GL_FLOAT, 0, nullptr);
+		glColorPointer(4, GL_FLOAT, 0, nullptr);
+	}
 }
 
 void lcContext::SetVertexBuffer(lcVertexBuffer VertexBuffer)
@@ -664,7 +818,7 @@ void lcContext::SetVertexBuffer(lcVertexBuffer VertexBuffer)
 	if (gSupportsVertexBufferObject)
 	{
 		GLuint VertexBufferObject = VertexBuffer.Object;
-		mVertexBufferPointer = NULL;
+		mVertexBufferPointer = nullptr;
 
 		if (VertexBufferObject != mVertexBufferObject)
 		{
@@ -692,58 +846,119 @@ void lcContext::SetVertexBufferPointer(const void* VertexBuffer)
 	mVertexBufferOffset = (char*)~0;
 }
 
-void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int TexCoordSize, int ColorSize)
+void lcContext::SetVertexFormatPosition(int PositionSize)
 {
-	int VertexSize = (PositionSize + TexCoordSize + ColorSize) * sizeof(float);
+	int VertexSize = PositionSize * sizeof(float);
+	char* VertexBufferPointer = mVertexBufferPointer;
+
+	if (gSupportsShaderObjects)
+	{
+		if (mVertexBufferOffset != mVertexBufferPointer)
+		{
+			glVertexAttribPointer(LC_ATTRIB_POSITION, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
+			mVertexBufferOffset = VertexBufferPointer;
+		}
+
+		if (mNormalEnabled)
+		{
+			glDisableVertexAttribArray(LC_ATTRIB_NORMAL);
+			mNormalEnabled = false;
+		}
+
+		if (mTexCoordEnabled)
+		{
+			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
+			mTexCoordEnabled = false;
+		}
+
+		if (mColorEnabled)
+		{
+			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
+			mColorEnabled = false;
+		}
+	}
+	else
+	{
+		if (mVertexBufferOffset != mVertexBufferPointer)
+		{
+			glVertexPointer(PositionSize, GL_FLOAT, VertexSize, VertexBufferPointer);
+			mVertexBufferOffset = VertexBufferPointer;
+		}
+
+		if (mNormalEnabled)
+		{
+			glDisableClientState(GL_NORMAL_ARRAY);
+			mNormalEnabled = false;
+		}
+
+		if (mTexCoordEnabled)
+		{
+			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+			mTexCoordEnabled = false;
+		}
+
+		if (mColorEnabled)
+		{
+			glDisableClientState(GL_COLOR_ARRAY);
+			mColorEnabled = false;
+		}
+	}
+}
+
+void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int NormalSize, int TexCoordSize, int ColorSize, bool EnableNormals)
+{
+	int VertexSize = (PositionSize + TexCoordSize + ColorSize) * sizeof(float) + NormalSize * sizeof(quint32);
 	char* VertexBufferPointer = mVertexBufferPointer + BufferOffset;
 
-	if (mVertexBufferOffset != VertexBufferPointer)
+	if (gSupportsShaderObjects)
 	{
-		if (gSupportsShaderObjects)
-			glVertexAttribPointer(LC_ATTRIB_POSITION, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
-		else
-			glVertexPointer(PositionSize, GL_FLOAT, VertexSize, VertexBufferPointer);
-
-		mVertexBufferOffset = VertexBufferPointer;
-	}
-
-	if (TexCoordSize)
-	{
-		if (gSupportsShaderObjects)
+		if (mVertexBufferOffset != VertexBufferPointer)
 		{
-			glVertexAttribPointer(LC_ATTRIB_TEXCOORD, TexCoordSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + PositionSize * sizeof(float));
+			glVertexAttribPointer(LC_ATTRIB_POSITION, PositionSize, GL_FLOAT, false, VertexSize, VertexBufferPointer);
+			mVertexBufferOffset = VertexBufferPointer;
+		}
+
+		int Offset = PositionSize * sizeof(float);
+
+		if (NormalSize && EnableNormals)
+		{
+			glVertexAttribPointer(LC_ATTRIB_NORMAL, 4, GL_BYTE, true, VertexSize, VertexBufferPointer + Offset);
+
+			if (!mNormalEnabled)
+			{
+				glEnableVertexAttribArray(LC_ATTRIB_NORMAL);
+				mNormalEnabled = true;
+			}
+		}
+		else if (mNormalEnabled)
+		{
+			glDisableClientState(GL_NORMAL_ARRAY);
+			mNormalEnabled = false;
+		}
+
+		Offset += NormalSize * sizeof(quint32);
+
+		if (TexCoordSize)
+		{
+			glVertexAttribPointer(LC_ATTRIB_TEXCOORD, TexCoordSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + Offset);
 
 			if (!mTexCoordEnabled)
 			{
 				glEnableVertexAttribArray(LC_ATTRIB_TEXCOORD);
 				mTexCoordEnabled = true;
 			}
-		}
-		else
-		{
-			glTexCoordPointer(TexCoordSize, GL_FLOAT, VertexSize, VertexBufferPointer + PositionSize * sizeof(float));
 
-			if (!mTexCoordEnabled)
-			{
-				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-				mTexCoordEnabled = true;
-			}
+			Offset += 2 * sizeof(float);
 		}
-	}
-	else if (mTexCoordEnabled)
-	{
-		if (gSupportsShaderObjects)
+		else if (mTexCoordEnabled)
+		{
 			glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-		else
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		mTexCoordEnabled = false;
-	}
+			mTexCoordEnabled = false;
+		}
 
-	if (ColorSize)
-	{
-		if (gSupportsShaderObjects)
+		if (ColorSize)
 		{
-			glVertexAttribPointer(LC_ATTRIB_COLOR, ColorSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + (PositionSize + TexCoordSize) * sizeof(float));
+			glVertexAttribPointer(LC_ATTRIB_COLOR, ColorSize, GL_FLOAT, false, VertexSize, VertexBufferPointer + Offset);
 
 			if (!mColorEnabled)
 			{
@@ -751,9 +966,61 @@ void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int TexCoord
 				mColorEnabled = true;
 			}
 		}
-		else
+		else if (mColorEnabled)
 		{
-			glColorPointer(ColorSize, GL_FLOAT, VertexSize, VertexBufferPointer + (PositionSize + TexCoordSize) * sizeof(float));
+			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
+			mColorEnabled = false;
+		}
+	}
+	else
+	{
+		if (mVertexBufferOffset != VertexBufferPointer)
+		{
+			glVertexPointer(PositionSize, GL_FLOAT, VertexSize, VertexBufferPointer);
+			mVertexBufferOffset = VertexBufferPointer;
+		}
+
+		int Offset = PositionSize * sizeof(float);
+
+		if (NormalSize && EnableNormals)
+		{
+			glNormalPointer(GL_BYTE, VertexSize, VertexBufferPointer + Offset);
+
+			if (!mNormalEnabled)
+			{
+				glEnableClientState(GL_NORMAL_ARRAY);
+				mNormalEnabled = true;
+			}
+		}
+		else if (mNormalEnabled)
+		{
+			glDisableClientState(GL_NORMAL_ARRAY);
+			mNormalEnabled = false;
+		}
+
+		Offset += NormalSize * sizeof(quint32);
+
+		if (TexCoordSize)
+		{
+			glTexCoordPointer(TexCoordSize, GL_FLOAT, VertexSize, VertexBufferPointer + Offset);
+
+			if (!mTexCoordEnabled)
+			{
+				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+				mTexCoordEnabled = true;
+			}
+
+			Offset += 2 * sizeof(float);
+		}
+		else if (mTexCoordEnabled)
+		{
+			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+			mTexCoordEnabled = false;
+		}
+
+		if (ColorSize)
+		{
+			glColorPointer(ColorSize, GL_FLOAT, VertexSize, VertexBufferPointer + Offset);
 
 			if (!mColorEnabled)
 			{
@@ -761,14 +1028,11 @@ void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int TexCoord
 				mColorEnabled = true;
 			}
 		}
-	}
-	else if (mColorEnabled)
-	{
-		if (gSupportsShaderObjects)
-			glDisableVertexAttribArray(LC_ATTRIB_COLOR);
-		else
+		else if (mColorEnabled)
+		{
 			glDisableClientState(GL_COLOR_ARRAY);
-		mColorEnabled = false;
+			mColorEnabled = false;
+		}
 	}
 }
 
@@ -786,7 +1050,7 @@ void lcContext::SetIndexBuffer(lcIndexBuffer IndexBuffer)
 	if (gSupportsVertexBufferObject)
 	{
 		GLuint IndexBufferObject = IndexBuffer.Object;
-		mIndexBufferPointer = NULL;
+		mIndexBufferPointer = nullptr;
 
 		if (IndexBufferObject != mIndexBufferObject)
 		{
@@ -811,7 +1075,7 @@ void lcContext::SetIndexBufferPointer(const void* IndexBuffer)
 	mIndexBufferPointer = (char*)IndexBuffer;
 }
 
-void lcContext::BindMesh(lcMesh* Mesh)
+void lcContext::BindMesh(const lcMesh* Mesh)
 {
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
 
@@ -824,7 +1088,7 @@ void lcContext::BindMesh(lcMesh* Mesh)
 		{
 			glBindBuffer(GL_ARRAY_BUFFER_ARB, VertexBufferObject);
 			mVertexBufferObject = VertexBufferObject;
-			mVertexBufferPointer = NULL;
+			mVertexBufferPointer = nullptr;
 			mVertexBufferOffset = (char*)~0;
 		}
 
@@ -832,7 +1096,7 @@ void lcContext::BindMesh(lcMesh* Mesh)
 		{
 			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, IndexBufferObject);
 			mIndexBufferObject = IndexBufferObject;
-			mIndexBufferPointer = NULL;
+			mIndexBufferPointer = nullptr;
 		}
 	}
 	else
@@ -855,44 +1119,12 @@ void lcContext::BindMesh(lcMesh* Mesh)
 	}
 }
 
-void lcContext::UnbindMesh()
-{
-	if (mTexture)
-	{
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glDisable(GL_TEXTURE_2D);
-		mTexture = NULL;
-	}
-
-	if (gSupportsVertexBufferObject)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER_ARB, 0);
-		mVertexBufferObject = 0;
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
-		mIndexBufferObject = 0;
-	}
-
-	mVertexBufferPointer = NULL;
-	mIndexBufferPointer = NULL;
-
-	if (gSupportsShaderObjects)
-	{
-		glDisableVertexAttribArray(LC_ATTRIB_TEXCOORD);
-		glDisableVertexAttribArray(LC_ATTRIB_COLOR);
-	}
-	else
-	{
-		glVertexPointer(3, GL_FLOAT, 0, NULL);
-		glTexCoordPointer(2, GL_FLOAT, 0, NULL);
-	}
-	mTexCoordEnabled = false;
-	mColorEnabled = false;
-}
-
 void lcContext::FlushState()
 {
 	if (gSupportsShaderObjects)
 	{
+		const lcProgram& Program = mPrograms[mMaterialType];
+
 		if (mWorldMatrixDirty || mViewMatrixDirty || mProjectionMatrixDirty)
 		{
 			if (mViewProjectionMatrixDirty)
@@ -901,15 +1133,36 @@ void lcContext::FlushState()
 				mViewProjectionMatrixDirty = false;
 			}
 
-			glUniformMatrix4fv(mPrograms[mProgramType].MatrixLocation, 1, false, lcMul(mWorldMatrix, mViewProjectionMatrix));
+			if (mWorldMatrixDirty)
+			{
+				if (Program.WorldMatrixLocation != -1)
+					glUniformMatrix4fv(Program.WorldMatrixLocation, 1, false, mWorldMatrix);
+			}
+
+			if (mViewMatrixDirty)
+			{
+				lcMatrix44 InverseViewMatrix = lcMatrix44AffineInverse(mViewMatrix);
+				lcVector3 ViewPosition = lcMul30(-mViewMatrix.GetTranslation(), InverseViewMatrix);
+
+				if (Program.LightPositionLocation != -1)
+				{
+					lcVector3 LightPosition = ViewPosition + lcMul30(lcVector3(300.0f, 300.0f, 0.0f), InverseViewMatrix);
+					glUniform3fv(Program.LightPositionLocation, 1, LightPosition);
+				}
+
+				if (Program.EyePositionLocation != -1)
+					glUniform3fv(Program.EyePositionLocation, 1, ViewPosition);
+			}
+
+			glUniformMatrix4fv(Program.WorldViewProjectionMatrixLocation, 1, false, lcMul(mWorldMatrix, mViewProjectionMatrix));
 			mWorldMatrixDirty = false;
 			mViewMatrixDirty = false;
 			mProjectionMatrixDirty = false;
 		}
 
-		if (mColorDirty && mPrograms[mProgramType].ColorLocation != -1)
+		if (mColorDirty && Program.MaterialColorLocation != -1)
 		{
-			glUniform4fv(mPrograms[mProgramType].ColorLocation, 1, mColor);
+			glUniform4fv(Program.MaterialColorLocation, 1, mColor);
 			mColorDirty = false;
 		}
 	}
@@ -957,221 +1210,3 @@ void lcContext::DrawIndexedPrimitives(GLenum Mode, GLsizei Count, GLenum Type, i
 	FlushState();
 	glDrawElements(Mode, Count, Type, mIndexBufferPointer + Offset);
 }
-
-void lcContext::DrawMeshSection(lcMesh* Mesh, lcMeshSection* Section)
-{
-	lcTexture* Texture = Section->Texture;
-	int VertexBufferOffset = Mesh->mVertexCacheOffset != -1 ? Mesh->mVertexCacheOffset : 0;
-	int IndexBufferOffset = Mesh->mIndexCacheOffset != -1 ? Mesh->mIndexCacheOffset : 0;
-
-	if (!Texture)
-	{
-		SetProgram(LC_PROGRAM_SIMPLE);
-		SetVertexFormat(VertexBufferOffset, 3, 0, 0);
-
-		if (mTexture)
-		{
-			glDisable(GL_TEXTURE_2D);
-			mTexture = NULL;
-		}
-	}
-	else
-	{
-		VertexBufferOffset += Mesh->mNumVertices * sizeof(lcVertex);
-		SetProgram(LC_PROGRAM_TEXTURE);
-		SetVertexFormat(VertexBufferOffset, 3, 2, 0);
-
-		if (Texture != mTexture)
-		{
-			glBindTexture(GL_TEXTURE_2D, Texture->mTexture);
-
-			if (!mTexture)
-			{
-				SetTextureMode(LC_TEXTURE_DECAL);
-				glEnable(GL_TEXTURE_2D);
-			}
-
-			mTexture = Texture;
-		}
-	}
-
-	bool DrawConditional = false;
-
-	if (Section->PrimitiveType != LC_MESH_CONDITIONAL_LINES)
-	{
-		GLenum PrimitiveType = (Section->PrimitiveType == LC_MESH_TRIANGLES || Section->PrimitiveType == LC_MESH_TEXTURED_TRIANGLES) ? GL_TRIANGLES : GL_LINES;
-		DrawIndexedPrimitives(PrimitiveType, Section->NumIndices, Mesh->mIndexType, IndexBufferOffset + Section->IndexOffset);
-	}
-	else if (DrawConditional)
-	{
-		FlushState();
-		lcMatrix44 WorldViewProjectionMatrix = lcMul(mWorldMatrix, mViewProjectionMatrix);
-		float* VertexBuffer = (float*)Mesh->mVertexData;
-
-		if (Mesh->mIndexType == GL_UNSIGNED_SHORT)
-		{
-			lcuint16* Indices = (lcuint16*)((char*)Mesh->mIndexData + Section->IndexOffset);
-
-			for (int i = 0; i < Section->NumIndices; i += 4)
-			{
-				lcVector3 p1 = lcMul31(lcVector3(VertexBuffer[Indices[i] * 3], VertexBuffer[Indices[i] * 3 + 1], VertexBuffer[Indices[i] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p2 = lcMul31(lcVector3(VertexBuffer[Indices[i + 1] * 3], VertexBuffer[Indices[i + 1] * 3 + 1], VertexBuffer[Indices[i + 1] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p3 = lcMul31(lcVector3(VertexBuffer[Indices[i + 2] * 3], VertexBuffer[Indices[i + 2] * 3 + 1], VertexBuffer[Indices[i + 2] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p4 = lcMul31(lcVector3(VertexBuffer[Indices[i + 3] * 3], VertexBuffer[Indices[i + 3] * 3 + 1], VertexBuffer[Indices[i + 3] * 3 + 2]), WorldViewProjectionMatrix);
-
-				if (((p1.y - p2.y) * (p3.x - p1.x) + (p2.x - p1.x) * (p3.y - p1.y)) * ((p1.y - p2.y) * (p4.x - p1.x) + (p2.x - p1.x) * (p4.y - p1.y)) >= 0)
-					DrawIndexedPrimitives(GL_LINES, 2, Mesh->mIndexType, IndexBufferOffset + Section->IndexOffset + i * sizeof(lcuint16));
-			}
-		}
-		else
-		{
-			lcuint32* Indices = (lcuint32*)((char*)Mesh->mIndexData + Section->IndexOffset);
-
-			for (int i = 0; i < Section->NumIndices; i += 4)
-			{
-				lcVector3 p1 = lcMul31(lcVector3(VertexBuffer[Indices[i] * 3], VertexBuffer[Indices[i] * 3 + 1], VertexBuffer[Indices[i] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p2 = lcMul31(lcVector3(VertexBuffer[Indices[i + 1] * 3], VertexBuffer[Indices[i + 1] * 3 + 1], VertexBuffer[Indices[i + 1] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p3 = lcMul31(lcVector3(VertexBuffer[Indices[i + 2] * 3], VertexBuffer[Indices[i + 2] * 3 + 1], VertexBuffer[Indices[i + 2] * 3 + 2]), WorldViewProjectionMatrix);
-				lcVector3 p4 = lcMul31(lcVector3(VertexBuffer[Indices[i + 3] * 3], VertexBuffer[Indices[i + 3] * 3 + 1], VertexBuffer[Indices[i + 3] * 3 + 2]), WorldViewProjectionMatrix);
-
-				if (((p1.y - p2.y) * (p3.x - p1.x) + (p2.x - p1.x) * (p3.y - p1.y)) * ((p1.y - p2.y) * (p4.x - p1.x) + (p2.x - p1.x) * (p4.y - p1.y)) >= 0)
-					DrawIndexedPrimitives(GL_LINES, 2, Mesh->mIndexType, IndexBufferOffset + Section->IndexOffset + i * sizeof(lcuint32));
-			}
-		}
-	}
-}
-
-void lcContext::DrawOpaqueMeshes(const lcArray<lcRenderMesh>& OpaqueMeshes)
-{
-	bool DrawLines = lcGetPreferences().mDrawEdgeLines;
-
-	lcGetPiecesLibrary()->UpdateBuffers(this); // TODO: find a better place for this update
-
-	for (int MeshIdx = 0; MeshIdx < OpaqueMeshes.GetSize(); MeshIdx++)
-	{
-		const lcRenderMesh& RenderMesh = OpaqueMeshes[MeshIdx];
-		lcMesh* Mesh = RenderMesh.Mesh;
-		int LodIndex = RenderMesh.LodIndex;
-
-		BindMesh(Mesh);
-		SetWorldMatrix(RenderMesh.WorldMatrix);
-
-		for (int SectionIdx = 0; SectionIdx < Mesh->mLods[LodIndex].NumSections; SectionIdx++)
-		{
-			lcMeshSection* Section = &Mesh->mLods[LodIndex].Sections[SectionIdx];
-			int ColorIndex = Section->ColorIndex;
-
-			if (Section->PrimitiveType == LC_MESH_TRIANGLES || Section->PrimitiveType == LC_MESH_TEXTURED_TRIANGLES)
-			{
-				if (ColorIndex == gDefaultColor)
-					ColorIndex = RenderMesh.ColorIndex;
-
-				if (lcIsColorTranslucent(ColorIndex))
-					continue;
-
-				switch (RenderMesh.State)
-				{
-				case LC_RENDERMESH_NONE:
-					SetColorIndex(ColorIndex);
-					break;
-
-				case LC_RENDERMESH_SELECTED:
-					SetColorIndexTinted(ColorIndex, LC_COLOR_SELECTED);
-					break;
-
-				case LC_RENDERMESH_FOCUSED:
-					SetColorIndexTinted(ColorIndex, LC_COLOR_FOCUSED);
-					break;
-				}
-			}
-			else
-			{
-				switch (RenderMesh.State)
-				{
-				case LC_RENDERMESH_NONE:
-					if (DrawLines)
-					{
-						if (ColorIndex == gEdgeColor)
-							SetEdgeColorIndex(RenderMesh.ColorIndex);
-						else
-							SetColorIndex(ColorIndex);
-					}
-					else
-						continue;
-					break;
-
-				case LC_RENDERMESH_SELECTED:
-					SetInterfaceColor(LC_COLOR_SELECTED);
-					break;
-
-				case LC_RENDERMESH_FOCUSED:
-					SetInterfaceColor(LC_COLOR_FOCUSED);
-					break;
-				}
-			}
-
-			DrawMeshSection(Mesh, Section);
-		}
-	}
-}
-
-void lcContext::DrawTranslucentMeshes(const lcArray<lcRenderMesh>& TranslucentMeshes)
-{
-	if (TranslucentMeshes.IsEmpty())
-		return;
-
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glEnable(GL_BLEND);
-	glDepthMask(GL_FALSE);
-
-	for (int MeshIdx = 0; MeshIdx < TranslucentMeshes.GetSize(); MeshIdx++)
-	{
-		const lcRenderMesh& RenderMesh = TranslucentMeshes[MeshIdx];
-		lcMesh* Mesh = RenderMesh.Mesh;
-		int LodIndex = RenderMesh.LodIndex;
-
-		BindMesh(Mesh);
-		SetWorldMatrix(RenderMesh.WorldMatrix);
-
-		for (int SectionIdx = 0; SectionIdx < Mesh->mLods[LodIndex].NumSections; SectionIdx++)
-		{
-			lcMeshSection* Section = &Mesh->mLods[LodIndex].Sections[SectionIdx];
-			int ColorIndex = Section->ColorIndex;
-
-			if (Section->PrimitiveType != LC_MESH_TRIANGLES)
-				continue;
-
-			if (ColorIndex == gDefaultColor)
-				ColorIndex = RenderMesh.ColorIndex;
-
-			if (!lcIsColorTranslucent(ColorIndex))
-				continue;
-
-			switch (RenderMesh.State)
-			{
-			case LC_RENDERMESH_NONE:
-				SetColorIndex(ColorIndex);
-				break;
-
-			case LC_RENDERMESH_SELECTED:
-				SetColorIndexTinted(ColorIndex, LC_COLOR_SELECTED);
-				break;
-
-			case LC_RENDERMESH_FOCUSED:
-				SetColorIndexTinted(ColorIndex, LC_COLOR_FOCUSED);
-				break;
-			}
-
-			DrawMeshSection(Mesh, Section);
-		}
-	}
-
-	glDepthMask(GL_TRUE);
-	glDisable(GL_BLEND);
-}
-
-void lcContext::DrawInterfaceObjects(const lcArray<const lcObject*>& InterfaceObjects)
-{
-	for (int ObjectIdx = 0; ObjectIdx < InterfaceObjects.GetSize(); ObjectIdx++)
-		InterfaceObjects[ObjectIdx]->DrawInterface(this);
-}
