@@ -21,6 +21,35 @@
 
 lcMainWindow* gMainWindow;
 
+void lcModelTabWidget::ResetLayout()
+{
+	QLayout* TabLayout = layout();
+	QWidget* TopWidget = TabLayout->itemAt(0)->widget();
+
+	if (TopWidget->metaObject() == &lcQGLWidget::staticMetaObject)
+		return;
+
+	QWidget* Widget = GetAnyViewWidget();
+
+	TabLayout->addWidget(Widget);
+	TabLayout->removeWidget(TopWidget);
+	TopWidget->deleteLater();
+
+	Widget->setFocus();
+	SetActiveView((View*)((lcQGLWidget*)Widget)->widget);
+}
+
+void lcModelTabWidget::Clear()
+{
+	ResetLayout();
+	mModel = nullptr;
+	mViews.RemoveAll();
+	mActiveView = nullptr;
+	lcQGLWidget* Widget = (lcQGLWidget*)layout()->itemAt(0)->widget();
+	delete Widget->widget;
+	Widget->widget = nullptr;
+}
+
 lcMainWindow::lcMainWindow()
 {
 	memset(mActions, 0, sizeof(mActions));
@@ -39,7 +68,7 @@ lcMainWindow::lcMainWindow()
 	mLockY = false;
 	mLockZ = false;
 	mRelativeTransform = true;
-	mCurrentPieceInfo = NULL;
+	mCurrentPieceInfo = nullptr;
 
 	memset(&mSearchOptions, 0, sizeof(mSearchOptions));
 
@@ -55,19 +84,19 @@ lcMainWindow::~lcMainWindow()
 	{
 		lcPiecesLibrary* Library = lcGetPiecesLibrary();
 		Library->ReleasePieceInfo(mCurrentPieceInfo);
-		mCurrentPieceInfo = NULL;
+		mCurrentPieceInfo = nullptr;
 	}
 
 	for (int FileIdx = 0; FileIdx < LC_MAX_RECENT_FILES; FileIdx++)
 		lcSetProfileString((LC_PROFILE_KEY)(LC_PROFILE_RECENT_FILE1 + FileIdx), mRecentFiles[FileIdx]);
 
-	gMainWindow = NULL;
+	gMainWindow = nullptr;
 }
 
 void lcMainWindow::CreateWidgets()
 {
 	setAcceptDrops(true);
-	setWindowIcon(QIcon(":/resources/icon64.png"));
+	setWindowIcon(QIcon(":/resources/leocad.png"));
 	setWindowFilePath(QString());
 
 	CreateActions();
@@ -75,6 +104,15 @@ void lcMainWindow::CreateWidgets()
 	CreateMenus();
 	CreateStatusBar();
 
+	int AASamples = lcGetProfileInt(LC_PROFILE_ANTIALIASING_SAMPLES);
+	if (AASamples > 1)
+	{
+		QGLFormat format;
+		format.setSampleBuffers(true);
+		format.setSamples(AASamples);
+		QGLFormat::setDefaultFormat(format);
+	}
+
 	mModelTabWidget = new QTabWidget();
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
     mModelTabWidget->tabBar()->setMovable(true);
@@ -340,6 +378,8 @@ void lcMainWindow::CreateMenus()
 	FileMenu->addAction(mActions[LC_FILE_SAVE]);
 	FileMenu->addAction(mActions[LC_FILE_SAVEAS]);
 	FileMenu->addAction(mActions[LC_FILE_SAVE_IMAGE]);
+	QMenu* ImportMenu = FileMenu->addMenu(tr("&Import"));
+	ImportMenu->addAction(mActions[LC_FILE_IMPORT_LDD]);
 	QMenu* ExportMenu = FileMenu->addMenu(tr("&Export"));
 	ExportMenu->addAction(mActions[LC_FILE_EXPORT_3DS]);
 	ExportMenu->addAction(mActions[LC_FILE_EXPORT_BRICKLINK]);
@@ -377,6 +417,7 @@ void lcMainWindow::CreateMenus()
 	EditMenu->addAction(mActions[LC_EDIT_SELECT_NONE]);
 	EditMenu->addAction(mActions[LC_EDIT_SELECT_INVERT]);
 	EditMenu->addAction(mActions[LC_EDIT_SELECT_BY_NAME]);
+	EditMenu->addAction(mActions[LC_EDIT_SELECT_BY_COLOR]);
 
 	QMenu* ViewMenu = menuBar()->addMenu(tr("&View"));
 	ViewMenu->addAction(mActions[LC_VIEW_PREFERENCES]);
@@ -416,6 +457,7 @@ void lcMainWindow::CreateMenus()
 	QMenu* PieceMenu = menuBar()->addMenu(tr("&Piece"));
 	PieceMenu->addAction(mActions[LC_PIECE_INSERT]);
 	PieceMenu->addAction(mActions[LC_PIECE_DELETE]);
+	PieceMenu->addAction(mActions[LC_PIECE_DUPLICATE]);
 	PieceMenu->addAction(mActions[LC_PIECE_RESET_PIVOT_POINT]);
 	PieceMenu->addAction(mActions[LC_PIECE_ARRAY]);
 	PieceMenu->addAction(mActions[LC_PIECE_MINIFIG_WIZARD]);
@@ -581,6 +623,7 @@ void lcMainWindow::CreateToolBars()
 
 	mColorsToolBar = new QDockWidget(tr("Colors"), this);
 	mColorsToolBar->setObjectName("ColorsToolbar");
+	mColorsToolBar->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
 
 	QFrame* ColorFrame = new QFrame(mColorsToolBar);
 	ColorFrame->setFrameShape(QFrame::StyledPanel);
@@ -694,7 +737,7 @@ void lcMainWindow::ModelTabChanged(int Index)
 	Project* Project = lcGetActiveProject();
 	lcModelTabWidget* CurrentTab = (lcModelTabWidget*)mModelTabWidget->widget(Index);
 
-	Project->SetActiveModel(Project->GetModels().FindIndex(CurrentTab ? CurrentTab->GetModel() : NULL));
+	Project->SetActiveModel(Project->GetModels().FindIndex(CurrentTab ? CurrentTab->GetModel() : nullptr));
 }
 
 void lcMainWindow::ClipboardChanged()
@@ -823,6 +866,7 @@ void lcMainWindow::Print(QPrinter* Printer)
 					free(Buffer);
 					Model->SetTemporaryStep(PreviousTime);
 					Context->EndRenderToTexture();
+					Context->ClearResources();
 					return;
 				}
 
@@ -994,6 +1038,7 @@ void lcMainWindow::Print(QPrinter* Printer)
 	Model->SetTemporaryStep(PreviousTime);
 
 	Context->EndRenderToTexture();
+	Context->ClearResources();
 #endif
 }
 
@@ -1068,15 +1113,23 @@ bool lcMainWindow::DoDialog(LC_DIALOG_TYPE Type, void* Data)
 
 void lcMainWindow::RemoveAllModelTabs()
 {
-	while (mModelTabWidget->count())
+	while (mModelTabWidget->count() > 1)
 	{
 		QWidget* TabWidget = mModelTabWidget->widget(0);
 		delete TabWidget;
 	}
+
+	if (mModelTabWidget->count())
+	{
+		lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->widget(0);
+		TabWidget->Clear();
+	}
 }
 
 void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 {
+	lcModelTabWidget* EmptyWidget = nullptr;
+
 	for (int TabIdx = 0; TabIdx < mModelTabWidget->count(); TabIdx++)
 	{
 		lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->widget(TabIdx);
@@ -1086,18 +1139,43 @@ void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 			mModelTabWidget->setCurrentIndex(TabIdx);
 			return;
 		}
+
+		if (!TabWidget->GetModel())
+			EmptyWidget = TabWidget;
 	}
 
-	lcModelTabWidget* TabWidget = new lcModelTabWidget(Model);
-	mModelTabWidget->addTab(TabWidget, Model->GetProperties().mName);
-	mModelTabWidget->setCurrentWidget(TabWidget);
+	lcModelTabWidget* TabWidget;
+	lcQGLWidget* ViewWidget;
+	View* NewView;
 
-	QGridLayout* CentralLayout = new QGridLayout(TabWidget);
-	CentralLayout->setContentsMargins(0, 0, 0, 0);
+	if (!EmptyWidget)
+	{
+		TabWidget = new lcModelTabWidget(Model);
+		mModelTabWidget->addTab(TabWidget, Model->GetProperties().mName);
+		mModelTabWidget->setCurrentWidget(TabWidget);
+
+		QGridLayout* CentralLayout = new QGridLayout(TabWidget);
+		CentralLayout->setContentsMargins(0, 0, 0, 0);
+
+		NewView = new View(Model);
+		ViewWidget = new lcQGLWidget(TabWidget, NewView, true);
+		CentralLayout->addWidget(ViewWidget, 0, 0, 1, 1);
+	}
+	else
+	{
+		TabWidget = EmptyWidget;
+		TabWidget->SetModel(Model);
+		mModelTabWidget->setCurrentWidget(TabWidget);
+
+		NewView = new View(Model);
+		ViewWidget = (lcQGLWidget*)TabWidget->layout()->itemAt(0)->widget();
+		ViewWidget->widget = NewView;
+		NewView->mWidget = ViewWidget;
+		NewView->mWidth = ViewWidget->width();
+		NewView->mHeight = ViewWidget->height();
+		AddView(NewView);
+	}
 
-	View* NewView = new View(Model);
-	QWidget* ViewWidget = new lcQGLWidget(TabWidget, NewView, true);
-	CentralLayout->addWidget(ViewWidget, 0, 0, 1, 1);
 	ViewWidget->show();
 	ViewWidget->setFocus();
 	NewView->ZoomExtents();
@@ -1327,6 +1405,7 @@ void lcMainWindow::SplitView(Qt::Orientation Orientation)
 	Sizes.append(10);
 	Sizes.append(10);
 	Splitter->setSizes(Sizes);
+	Focus->setFocus();
 }
 
 void lcMainWindow::SplitHorizontal()
@@ -1354,46 +1433,49 @@ void lcMainWindow::RemoveActiveView()
 	QWidget* ParentParentWidget = Parent->parentWidget();
 	QSplitter* ParentSplitter = (QSplitter*)Parent;
 	int FocusIndex = ParentSplitter->indexOf(Focus);
+	QWidget* OtherWidget = ParentSplitter->widget(!FocusIndex);
 
 	if (ParentParentWidget->metaObject() == &lcModelTabWidget::staticMetaObject)
 	{
 		QLayout* CentralLayout = ParentParentWidget->layout();
 
-		CentralLayout->addWidget(ParentSplitter->widget(!FocusIndex));
+		CentralLayout->addWidget(OtherWidget);
 		CentralLayout->removeWidget(Parent);
+	}
+	else
+	{
+		QSplitter* ParentParentSplitter = (QSplitter*)ParentParentWidget;
+		QList<int> Sizes = ParentParentSplitter->sizes();
 
-		return;
+		int ParentIndex = ParentParentSplitter->indexOf(Parent);
+		ParentParentSplitter->insertWidget(ParentIndex, OtherWidget);
+
+		ParentParentSplitter->setSizes(Sizes);
 	}
 
-	QSplitter* ParentParentSplitter = (QSplitter*)ParentParentWidget;
-	QList<int> Sizes = ParentParentSplitter->sizes();
+	Parent->deleteLater();
 
-	int ParentIndex = ParentParentSplitter->indexOf(Parent);
-	ParentParentSplitter->insertWidget(!ParentIndex, Focus);
+	if (OtherWidget->metaObject() != &lcQGLWidget::staticMetaObject)
+	{
+		lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->currentWidget();
 
-	delete Parent;
+		if (TabWidget)
+			OtherWidget = TabWidget->GetAnyViewWidget();
+	}
 
-	ParentParentSplitter->setSizes(Sizes);
+	OtherWidget->setFocus();
+	SetActiveView((View*)((lcQGLWidget*)OtherWidget)->widget);
 }
 
 void lcMainWindow::ResetViews()
 {
-	QWidget* TabWidget = mModelTabWidget->currentWidget();
+	lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->currentWidget();
 
 	if (!TabWidget)
 		return;
 
-	QLayout* TabLayout = TabWidget->layout();
-	QWidget* TopWidget = TabLayout->itemAt(0)->widget();
-	TabLayout->removeWidget(TopWidget);
-	TopWidget->deleteLater();
-	View* NewView = new View(lcGetActiveModel());
-	QWidget* ViewWidget = new lcQGLWidget(TabWidget, NewView, true);
-	TabLayout->addWidget(ViewWidget);
-	ViewWidget->show();
-	ViewWidget->setFocus();
-	NewView->ZoomExtents();
-	SetActiveView(NewView);
+	TabWidget->ResetLayout();
+	TabWidget->GetActiveView()->SetViewpoint(LC_VIEWPOINT_HOME);
 }
 
 void lcMainWindow::TogglePrintPreview()
@@ -1474,10 +1556,12 @@ void lcMainWindow::UpdateSelectedObjects(bool SelectionChanged)
 		mActions[LC_EDIT_FIND_PREVIOUS]->setEnabled((Flags & LC_SEL_NO_PIECES) == 0);
 		mActions[LC_EDIT_SELECT_INVERT]->setEnabled((Flags & LC_SEL_NO_PIECES) == 0);
 		mActions[LC_EDIT_SELECT_BY_NAME]->setEnabled((Flags & LC_SEL_NO_PIECES) == 0);
+		mActions[LC_EDIT_SELECT_BY_COLOR]->setEnabled((Flags & LC_SEL_NO_PIECES) == 0);
 		mActions[LC_EDIT_SELECT_NONE]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_EDIT_SELECT_ALL]->setEnabled(Flags & LC_SEL_UNSELECTED);
 
 		mActions[LC_PIECE_DELETE]->setEnabled(Flags & LC_SEL_SELECTED);
+		mActions[LC_PIECE_DUPLICATE]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_PIECE_RESET_PIVOT_POINT]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_PIECE_ARRAY]->setEnabled(Flags & LC_SEL_PIECE);
 		mActions[LC_PIECE_CONTROL_POINT_INSERT]->setEnabled(Flags & LC_SEL_CAN_ADD_CONTROL_POINT);
@@ -1626,7 +1710,7 @@ void lcMainWindow::UpdateCameraMenu()
 {
 	const lcArray<lcCamera*>& Cameras = lcGetActiveModel()->GetCameras();
 	View* ActiveView = GetActiveView();
-	lcCamera* CurrentCamera = ActiveView ? ActiveView->mCamera : NULL;
+	lcCamera* CurrentCamera = ActiveView ? ActiveView->mCamera : nullptr;
 	int CurrentIndex = -1;
 
 	for (int ActionIdx = LC_VIEW_CAMERA_FIRST; ActionIdx <= LC_VIEW_CAMERA_LAST; ActionIdx++)
@@ -1697,7 +1781,9 @@ void lcMainWindow::UpdateModels()
 		lcModelTabWidget* TabWidget = (lcModelTabWidget*)mModelTabWidget->widget(TabIdx);
 		lcModel* Model = TabWidget->GetModel();
 
-		if (Models.FindIndex(Model) != -1)
+		if (!Model)
+			TabIdx++;
+		else if (Models.FindIndex(Model) != -1)
 		{
 			mModelTabWidget->setTabText(TabIdx, Model->GetProperties().mName);
 			TabIdx++;
@@ -1709,7 +1795,7 @@ void lcMainWindow::UpdateModels()
 	mPartSelectionWidget->UpdateModels();
 
 	if (mCurrentPieceInfo && mCurrentPieceInfo->IsModel())
-		SetCurrentPieceInfo(NULL);
+		SetCurrentPieceInfo(nullptr);
 }
 
 void lcMainWindow::UpdateCategories()
@@ -1838,6 +1924,26 @@ void lcMainWindow::MergeProject()
 	}
 }
 
+void lcMainWindow::ImportLDD()
+{
+	if (!SaveProjectIfModified())
+		return;
+
+	QString LoadFileName = QFileDialog::getOpenFileName(this, tr("Import"), QString(), tr("LEGO Diginal Designer Files (*.lxf);;All Files (*.*)"));
+	if (LoadFileName.isEmpty())
+		return;
+
+	Project* NewProject = new Project();
+
+	if (NewProject->ImportLDD(LoadFileName))
+	{
+		g_App->SetProject(NewProject);
+		UpdateAllViews();
+	}
+	else
+		delete NewProject;
+}
+
 bool lcMainWindow::SaveProject(const QString& FileName)
 {
 	QString SaveFileName;
@@ -1964,6 +2070,10 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		lcGetActiveProject()->SaveImage();
 		break;
 
+	case LC_FILE_IMPORT_LDD:
+		ImportLDD();
+		break;
+
 	case LC_FILE_EXPORT_3DS:
 		lcGetActiveProject()->Export3DStudio(QString());
 		break;
@@ -2061,6 +2171,10 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		lcGetActiveModel()->ShowSelectByNameDialog();
 		break;
 
+	case LC_EDIT_SELECT_BY_COLOR:
+		lcGetActiveModel()->ShowSelectByColorDialog();
+		break;
+
 	case LC_VIEW_SPLIT_HORIZONTAL:
 		SplitHorizontal();
 		break;
@@ -2099,6 +2213,10 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		lcGetActiveModel()->DeleteSelectedObjects();
 		break;
 
+	case LC_PIECE_DUPLICATE:
+		lcGetActiveModel()->DuplicateSelectedPieces();
+		break;
+
 	case LC_PIECE_RESET_PIVOT_POINT:
 		lcGetActiveModel()->ResetSelectedPiecesPivotPoint();
 		break;
