@@ -52,11 +52,6 @@ public:
 	{
 	}
 
-	lcVector2(const lcVector2& a)
-		: x(a.x), y(a.y)
-	{
-	}
-
 	operator const float*() const
 	{
 		return (const float*)this;
@@ -253,6 +248,8 @@ public:
 		return r[i];
 	}
 
+	float Determinant() const;
+
 	lcVector4 r[4];
 };
 
@@ -530,6 +527,28 @@ inline lcVector4& operator/=(lcVector4& a, float b)
 	return a;
 }
 
+inline qint32 lcPackNormal(const lcVector3& Normal)
+{
+	quint32 Packed = 0;
+
+	Packed |= (((qint8)(Normal.x * 127.0f)) & 0xff) << 0;
+	Packed |= (((qint8)(Normal.y * 127.0f)) & 0xff) << 8;
+	Packed |= (((qint8)(Normal.z * 127.0f)) & 0xff) << 16;
+
+	return Packed;
+}
+
+inline lcVector3 lcUnpackNormal(qint32 Packed)
+{
+	lcVector3 Normal;
+
+	Normal.x = (float)(qint8)((Packed >>  0) & 0xff) / 127.0f;
+	Normal.y = (float)(qint8)((Packed >>  8) & 0xff) / 127.0f;
+	Normal.z = (float)(qint8)((Packed >> 16) & 0xff) / 127.0f;
+
+	return Normal;
+}
+
 inline lcVector3 lcVector3LDrawToLeoCAD(const lcVector3& Vector)
 {
 	return lcVector3(Vector[0], Vector[2], -Vector[1]);
@@ -798,6 +817,13 @@ inline lcVector3 lcMatrix33ToEulerAngles(const lcMatrix33& RotMat)
 	return Rot;
 }
 
+inline float lcMatrix44::Determinant() const
+{
+	return r[0][0] * r[1][1] * r[2][2] + r[0][1] * r[1][2] * r[2][0] +
+	       r[0][2] * r[1][0] * r[2][1] - r[0][0] * r[1][2] * r[2][1] -
+	       r[0][1] * r[1][0] * r[2][2] - r[0][2] * r[1][1] * r[2][0];
+}
+
 inline lcMatrix44 lcMatrix44Identity()
 {
 	lcMatrix44 m;
@@ -994,12 +1020,7 @@ inline lcVector4 lcMatrix44ToAxisAngle(const lcMatrix44& m)
 	Rows[1] = lcNormalize(lcVector3(m.r[1][0], m.r[1][1], m.r[1][2]));
 	Rows[2] = lcNormalize(lcVector3(m.r[2][0], m.r[2][1], m.r[2][2]));
 
-	// Determinant should be 1 for rotation matrices.
-	float Determinant = Rows[0][0] * Rows[1][1] * Rows[2][2] + Rows[0][1] * Rows[1][2] * Rows[2][0] +
-	                    Rows[0][2] * Rows[1][0] * Rows[2][1] - Rows[0][0] * Rows[1][2] * Rows[2][1] - 
-	                    Rows[0][1] * Rows[1][0] * Rows[2][2] - Rows[0][2] * Rows[1][1] * Rows[2][0];
-
-	if (Determinant < 0.0f)
+	if (m.Determinant() < 0.0f)
 		Rows[0] *= -1.0f;
 
 	float Trace = Rows[0][0] + Rows[1][1] + Rows[2][2];
@@ -1648,10 +1669,10 @@ inline void lcPolygonPlaneClip(lcVector3* InPoints, int NumInPoints, lcVector3*
 }
 
 // Return true if a polygon intersects a set of planes.
-inline bool lcTriangleIntersectsPlanes(float* p1, float* p2, float* p3, const lcVector4 Planes[6])
+inline bool lcTriangleIntersectsPlanes(const float* p1, const float* p2, const float* p3, const lcVector4 Planes[6])
 {
 	const int NumPlanes = 6;
-	float* Points[3] = { p1, p2, p3 };
+	const float* Points[3] = { p1, p2, p3 };
 	int Outcodes[3] = { 0, 0, 0 }, i;
 	int NumPoints = 3;
 
@@ -1725,6 +1746,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 		else
 		{
 			MiddleQuadrant[i] = true;
+			CandidatePlane[i] = 0.0f;
 		}
 	}
 
@@ -1733,7 +1755,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 	{
 		*Dist = 0;
 
-		if (*Intersection)
+		if (Intersection)
 			*Intersection = Start;
 
 		return true;
@@ -1776,7 +1798,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 
 	*Dist = lcLength(Point - Start);
 
-	if (*Intersection)
+	if (Intersection)
 		*Intersection = Point;
 
 	return true;
