@@ -1,13 +1,12 @@
-#ifndef _LC_MATH_H_
-#define _LC_MATH_H_
+#pragma once
 
 #include <math.h>
 #include <float.h>
 
-#define LC_DTOR 0.017453f
-#define LC_RTOD 57.29578f
-#define LC_PI   3.141592f
-#define LC_2PI  6.283185f
+#define LC_DTOR (static_cast<float>(M_PI / 180))
+#define LC_RTOD (static_cast<float>(180 / M_PI))
+#define LC_PI (static_cast<float>(M_PI))
+#define LC_2PI (static_cast<float>(2 * M_PI))
 
 #define LC_RGB(r,g,b) LC_RGBA(r,g,b,255)
 #define LC_RGBA(r,g,b,a) ((lcuint32)(((lcuint8) (r) | ((lcuint16) (g) << 8)) | (((lcuint32) (lcuint8) (b)) << 16) | (((lcuint32) (lcuint8) (a)) << 24))) 
@@ -364,9 +363,32 @@ inline bool operator!=(const lcVector3& a, const lcVector3& b)
 
 #ifndef QT_NO_DEBUG
 
-inline QDebug operator<<(QDebug d, const lcVector3& v)
+inline QDebug operator<<(QDebug Debug, const lcVector3& v)
 {
-	return d << v.x << v.y << v.z;
+	QDebugStateSaver Saver(Debug);
+	Debug.nospace() << '(' << v.x << ", " << v.y << ", " << v.z << ')';
+	return Debug;
+}
+
+inline QDebug operator<<(QDebug Debug, const lcVector4& v)
+{
+	QDebugStateSaver Saver(Debug);
+	Debug.nospace() << '(' << v.x << ", " << v.y << ", " << v.z << ", " << v.w << ')';
+	return Debug;
+}
+
+inline QDebug operator<<(QDebug Debug, const lcMatrix33& m)
+{
+	QDebugStateSaver Saver(Debug);
+	Debug.nospace() << '[' << m[0] << ", " << m[1] << ", " << m[2] << ']';
+	return Debug;
+}
+
+inline QDebug operator<<(QDebug Debug, const lcMatrix44& m)
+{
+	QDebugStateSaver Saver(Debug);
+	Debug.nospace() << '[' << m[0] << ", " << m[1] << ", " << m[2] << ", " << m[3] << ']';
+	return Debug;
 }
 
 #endif
@@ -755,6 +777,17 @@ inline lcMatrix33 lcMatrix33FromAxisAngle(const lcVector3& Axis, const float Rad
 	return m;
 }
 
+inline lcMatrix33 lcMatrix33Transpose(const lcMatrix33& m)
+{
+	lcMatrix33 t;
+
+	t.r[0] = lcVector3(m[0][0], m[1][0], m[2][0]);
+	t.r[1] = lcVector3(m[0][1], m[1][1], m[2][1]);
+	t.r[2] = lcVector3(m[0][2], m[1][2], m[2][2]);
+
+	return t;
+}
+
 inline lcMatrix33 lcMatrix33AffineInverse(const lcMatrix33& m)
 {
 	lcMatrix33 Inv;
@@ -1309,9 +1342,9 @@ inline lcMatrix44 lcMatrix44LDrawToLeoCAD(const lcMatrix44& Matrix)
 {
 	lcMatrix44 m;
 
-	m.r[0] = lcVector4(Matrix[0][0], Matrix[2][0], -Matrix[1][0], 0.0f);
-	m.r[1] = lcVector4(Matrix[0][2], Matrix[2][2], -Matrix[1][2], 0.0f);
-	m.r[2] = lcVector4(-Matrix[0][1], -Matrix[2][1], Matrix[1][1], 0.0f);
+	m.r[0] = lcVector4(Matrix[0][0], Matrix[0][2], -Matrix[0][1], 0.0f);
+	m.r[1] = lcVector4(Matrix[2][0], Matrix[2][2], -Matrix[2][1], 0.0f);
+	m.r[2] = lcVector4(-Matrix[1][0], -Matrix[1][2], Matrix[1][1], 0.0f);
 	m.r[3] = lcVector4(Matrix[3][0], Matrix[3][2], -Matrix[3][1], 1.0f);
 
 	return m;
@@ -1746,6 +1779,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 		else
 		{
 			MiddleQuadrant[i] = true;
+			CandidatePlane[i] = 0.0f;
 		}
 	}
 
@@ -1754,7 +1788,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 	{
 		*Dist = 0;
 
-		if (*Intersection)
+		if (Intersection)
 			*Intersection = Start;
 
 		return true;
@@ -1797,7 +1831,7 @@ inline bool lcBoundingBoxRayIntersectDistance(const lcVector3& Min, const lcVect
 
 	*Dist = lcLength(Point - Start);
 
-	if (*Intersection)
+	if (Intersection)
 		*Intersection = Point;
 
 	return true;
@@ -1951,4 +1985,3 @@ bool SphereIntersectsVolume(const Vector3& Center, float Radius, const Vector4*
 	return true;
 }*/
 
-#endif // _LC_MATH_H_
