@@ -13,16 +13,16 @@ lcMesh::lcMesh()
 {
 	for (int LodIdx = 0; LodIdx < LC_NUM_MESH_LODS; LodIdx++)
 	{
-		mLods[LodIdx].Sections = NULL;
+		mLods[LodIdx].Sections = nullptr;
 		mLods[LodIdx].NumSections = 0;
 	}
 
 	mNumVertices = 0;
 	mNumTexturedVertices = 0;
 	mIndexType = 0;
-	mVertexData = NULL;
+	mVertexData = nullptr;
 	mVertexDataSize = 0;
-	mIndexData = NULL;
+	mIndexData = nullptr;
 	mIndexDataSize = 0;
 	mVertexCacheOffset = -1;
 	mIndexCacheOffset = -1;
@@ -70,7 +70,7 @@ void lcMesh::CreateBox()
 	memset(NumSections, 0, sizeof(NumSections));
 	NumSections[LC_MESH_LOD_HIGH] = 2;
 
-	Create(NumSections, 8, 0, 36 + 24);
+	Create(NumSections, 24, 0, 36 + 24);
 
 	lcVector3 Min(-10.0f, -10.0f, -24.0f);
 	lcVector3 Max(10.0f, 10.0f, 4.0f);
@@ -78,24 +78,66 @@ void lcMesh::CreateBox()
 	mBoundingBox.Min = Min;
 	mBoundingBox.Max = Max;
 
-	float* Verts = (float*)mVertexData;
+	lcVertex* Verts = (lcVertex*)mVertexData;
 	lcuint16* Indices = (lcuint16*)mIndexData;
 
-	*Verts++ = Min[0]; *Verts++ = Min[1]; *Verts++ = Min[2];
-	*Verts++ = Min[0]; *Verts++ = Max[1]; *Verts++ = Min[2];
-	*Verts++ = Max[0]; *Verts++ = Max[1]; *Verts++ = Min[2];
-	*Verts++ = Max[0]; *Verts++ = Min[1]; *Verts++ = Min[2];
-	*Verts++ = Min[0]; *Verts++ = Min[1]; *Verts++ = Max[2];
-	*Verts++ = Min[0]; *Verts++ = Max[1]; *Verts++ = Max[2];
-	*Verts++ = Max[0]; *Verts++ = Max[1]; *Verts++ = Max[2];
-	*Verts++ = Max[0]; *Verts++ = Min[1]; *Verts++ = Max[2];
+	Verts[0].Position = lcVector3(Min[0], Min[1], Min[2]);
+	Verts[0].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, -1.0f));
+	Verts[1].Position = lcVector3(Min[0], Max[1], Min[2]);
+	Verts[1].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, -1.0f));
+	Verts[2].Position = lcVector3(Max[0], Max[1], Min[2]);
+	Verts[2].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, -1.0f));
+	Verts[3].Position = lcVector3(Max[0], Min[1], Min[2]);
+	Verts[3].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, -1.0f));
+	Verts[4].Position = lcVector3(Min[0], Min[1], Max[2]);
+	Verts[4].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, 1.0f));
+	Verts[5].Position = lcVector3(Min[0], Max[1], Max[2]);
+	Verts[5].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, 1.0f));
+	Verts[6].Position = lcVector3(Max[0], Max[1], Max[2]);
+	Verts[6].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, 1.0f));
+	Verts[7].Position = lcVector3(Max[0], Min[1], Max[2]);
+	Verts[7].Normal = lcPackNormal(lcVector3(0.0f, 0.0f, 1.0f));
+
+	Verts[8].Position = lcVector3(Min[0], Min[1], Min[2]);
+	Verts[8].Normal = lcPackNormal(lcVector3(-1.0f, 0.0f, 0.0f));
+	Verts[9].Position = lcVector3(Min[0], Min[1], Max[2]);
+	Verts[9].Normal = lcPackNormal(lcVector3(-1.0f, 0.0f, 0.0f));
+	Verts[10].Position = lcVector3(Min[0], Max[1], Max[2]);
+	Verts[10].Normal = lcPackNormal(lcVector3(-1.0f, 0.0f, 0.0f));
+	Verts[11].Position = lcVector3(Min[0], Max[1], Min[2]);
+	Verts[11].Normal = lcPackNormal(lcVector3(-1.0f, 0.0f, 0.0f));
+	Verts[12].Position = lcVector3(Max[0], Min[1], Min[2]);
+	Verts[12].Normal = lcPackNormal(lcVector3(1.0f, 0.0f, 0.0f));
+	Verts[13].Position = lcVector3(Max[0], Min[1], Max[2]);
+	Verts[13].Normal = lcPackNormal(lcVector3(1.0f, 0.0f, 0.0f));
+	Verts[14].Position = lcVector3(Max[0], Max[1], Max[2]);
+	Verts[14].Normal = lcPackNormal(lcVector3(1.0f, 0.0f, 0.0f));
+	Verts[15].Position = lcVector3(Max[0], Max[1], Min[2]);
+	Verts[15].Normal = lcPackNormal(lcVector3(1.0f, 0.0f, 0.0f));
+
+	Verts[16].Position = lcVector3(Min[0], Min[1], Min[2]);
+	Verts[16].Normal = lcPackNormal(lcVector3(0.0f, -1.0f, 0.0f));
+	Verts[17].Position = lcVector3(Min[0], Min[1], Max[2]);
+	Verts[17].Normal = lcPackNormal(lcVector3(0.0f, -1.0f, 0.0f));
+	Verts[18].Position = lcVector3(Max[0], Min[1], Max[2]);
+	Verts[18].Normal = lcPackNormal(lcVector3(0.0f, -1.0f, 0.0f));
+	Verts[19].Position = lcVector3(Max[0], Min[1], Min[2]);
+	Verts[19].Normal = lcPackNormal(lcVector3(0.0f, -1.0f, 0.0f));
+	Verts[20].Position = lcVector3(Min[0], Max[1], Min[2]);
+	Verts[20].Normal = lcPackNormal(lcVector3(0.0f, 1.0f, 0.0f));
+	Verts[21].Position = lcVector3(Min[0], Max[1], Max[2]);
+	Verts[21].Normal = lcPackNormal(lcVector3(0.0f, 1.0f, 0.0f));
+	Verts[22].Position = lcVector3(Max[0], Max[1], Max[2]);
+	Verts[22].Normal = lcPackNormal(lcVector3(0.0f, 1.0f, 0.0f));
+	Verts[23].Position = lcVector3(Max[0], Max[1], Min[2]);
+	Verts[23].Normal = lcPackNormal(lcVector3(0.0f, 1.0f, 0.0f));
 
 	lcMeshSection* Section = &mLods[LC_MESH_LOD_HIGH].Sections[0];
 	Section->ColorIndex = gDefaultColor;
 	Section->IndexOffset = 0;
 	Section->NumIndices = 36;
 	Section->PrimitiveType = LC_MESH_TRIANGLES;
-	Section->Texture = NULL;
+	Section->Texture = nullptr;
 
 	*Indices++ = 0; *Indices++ = 1; *Indices++ = 2;
 	*Indices++ = 0; *Indices++ = 2; *Indices++ = 3;
@@ -103,24 +145,25 @@ void lcMesh::CreateBox()
 	*Indices++ = 7; *Indices++ = 6; *Indices++ = 5;
 	*Indices++ = 7; *Indices++ = 5; *Indices++ = 4;
 
-	*Indices++ = 0; *Indices++ = 1; *Indices++ = 5;
-	*Indices++ = 0; *Indices++ = 5; *Indices++ = 4;
+	*Indices++ = 8; *Indices++ = 9; *Indices++ = 10;
+	*Indices++ = 8; *Indices++ = 10; *Indices++ = 11;
 
-	*Indices++ = 2; *Indices++ = 3; *Indices++ = 7;
-	*Indices++ = 2; *Indices++ = 7; *Indices++ = 6;
+	*Indices++ = 15; *Indices++ = 14; *Indices++ = 13;
+	*Indices++ = 15; *Indices++ = 13; *Indices++ = 12;
 
-	*Indices++ = 0; *Indices++ = 3; *Indices++ = 7;
-	*Indices++ = 0; *Indices++ = 7; *Indices++ = 4;
+	*Indices++ = 16; *Indices++ = 17; *Indices++ = 18;
+	*Indices++ = 16; *Indices++ = 18; *Indices++ = 19;
+
+	*Indices++ = 23; *Indices++ = 22; *Indices++ = 21;
+	*Indices++ = 23; *Indices++ = 21; *Indices++ = 20;
 
-	*Indices++ = 1; *Indices++ = 2; *Indices++ = 6;
-	*Indices++ = 1; *Indices++ = 6; *Indices++ = 5;
 
 	Section = &mLods[LC_MESH_LOD_HIGH].Sections[1];
 	Section->ColorIndex = gEdgeColor;
 	Section->IndexOffset = 36 * 2;
 	Section->NumIndices = 24;
 	Section->PrimitiveType = LC_MESH_LINES;
-	Section->Texture = NULL;
+	Section->Texture = nullptr;
 
 	*Indices++ = 0; *Indices++ = 1; *Indices++ = 1; *Indices++ = 2;
 	*Indices++ = 2; *Indices++ = 3; *Indices++ = 3; *Indices++ = 0;
@@ -136,10 +179,10 @@ template<typename IndexType>
 bool lcMesh::MinIntersectDist(const lcVector3& Start, const lcVector3& End, float& MinDistance)
 {
 	float Distance;
-	if (!lcBoundingBoxRayIntersectDistance(mBoundingBox.Min, mBoundingBox.Max, Start, End, &Distance, NULL) || (Distance >= MinDistance))
+	if (!lcBoundingBoxRayIntersectDistance(mBoundingBox.Min, mBoundingBox.Max, Start, End, &Distance, nullptr) || (Distance >= MinDistance))
 		return false;
 
-	float* Verts = (float*)mVertexData;
+	lcVertex* Verts = (lcVertex*)mVertexData;
 	bool Hit = false;
 	lcVector3 Intersection;
 
@@ -154,12 +197,9 @@ bool lcMesh::MinIntersectDist(const lcVector3& Start, const lcVector3& End, floa
 
 		for (int Idx = 0; Idx < Section->NumIndices; Idx += 3)
 		{
-			float* p1 = Verts + Indices[Idx + 0] * 3;
-			float* p2 = Verts + Indices[Idx + 1] * 3;
-			float* p3 = Verts + Indices[Idx + 2] * 3;
-			lcVector3 v1(p1[0], p1[1], p1[2]);
-			lcVector3 v2(p2[0], p2[1], p2[2]);
-			lcVector3 v3(p3[0], p3[1], p3[2]);
+			const lcVector3& v1 = Verts[Indices[Idx]].Position;
+			const lcVector3& v2 = Verts[Indices[Idx + 1]].Position;
+			const lcVector3& v3 = Verts[Indices[Idx + 2]].Position;
 
 			if (lcLineTriangleMinIntersection(v1, v2, v3, Start, End, &MinDistance, &Intersection))
 				Hit = true;
@@ -180,7 +220,7 @@ bool lcMesh::MinIntersectDist(const lcVector3& Start, const lcVector3& End, floa
 template<typename IndexType>
 bool lcMesh::IntersectsPlanes(const lcVector4 Planes[6])
 {
-	float* Verts = (float*)mVertexData;
+	lcVertex* Verts = (lcVertex*)mVertexData;
 
 	for (int SectionIdx = 0; SectionIdx < mLods[LC_MESH_LOD_HIGH].NumSections; SectionIdx++)
 	{
@@ -192,7 +232,7 @@ bool lcMesh::IntersectsPlanes(const lcVector4 Planes[6])
 		IndexType* Indices = (IndexType*)mIndexData + Section->IndexOffset / sizeof(IndexType);
 
 		for (int Idx = 0; Idx < Section->NumIndices; Idx += 3)
-			if (lcTriangleIntersectsPlanes(&Verts[Indices[Idx]*3], &Verts[Indices[Idx+1]*3], &Verts[Indices[Idx+2]*3], Planes))
+			if (lcTriangleIntersectsPlanes(Verts[Indices[Idx]].Position, Verts[Indices[Idx+1]].Position, Verts[Indices[Idx+2]].Position, Planes))
 				return true;
 	}
 
@@ -215,7 +255,7 @@ void lcMesh::ExportPOVRay(lcFile& File, const char* MeshName, const char** Color
 	sprintf(Line, "#declare lc_%s = union {\n", MeshName);
 	File.WriteLine(Line);
 
-	float* Verts = (float*)mVertexData;
+	lcVertex* Verts = (lcVertex*)mVertexData;
 
 	for (int SectionIdx = 0; SectionIdx < mLods[LC_MESH_LOD_HIGH].NumSections; SectionIdx++)
 	{
@@ -230,10 +270,11 @@ void lcMesh::ExportPOVRay(lcFile& File, const char* MeshName, const char** Color
 
 		for (int Idx = 0; Idx < Section->NumIndices; Idx += 3)
 		{
-			sprintf(Line, "  triangle { <%.2f, %.2f, %.2f>, <%.2f, %.2f, %.2f>, <%.2f, %.2f, %.2f> }\n",
-				-Verts[Indices[Idx+0]*3+1] / 25.0f, -Verts[Indices[Idx+0]*3] / 25.0f, Verts[Indices[Idx+0]*3+2] / 25.0f,
-				-Verts[Indices[Idx+1]*3+1] / 25.0f, -Verts[Indices[Idx+1]*3] / 25.0f, Verts[Indices[Idx+1]*3+2] / 25.0f,
-				-Verts[Indices[Idx+2]*3+1] / 25.0f, -Verts[Indices[Idx+2]*3] / 25.0f, Verts[Indices[Idx+2]*3+2] / 25.0f);
+			lcVector3& v1 = Verts[Indices[Idx]].Position;
+			lcVector3& v2 = Verts[Indices[Idx + 1]].Position;
+			lcVector3& v3 = Verts[Indices[Idx + 2]].Position;
+
+			sprintf(Line, "  triangle { <%.2f, %.2f, %.2f>, <%.2f, %.2f, %.2f>, <%.2f, %.2f, %.2f> }\n", -v1.y / 25.0f, -v1.x / 25.0f, v1.z / 25.0f, -v2.y / 25.0f, -v2.x / 25.0f, v2.z / 25.0f, -v3.y / 25.0f, -v3.x / 25.0f, v3.z / 25.0f);
 			File.WriteLine(Line);
 		}
 
@@ -348,14 +389,14 @@ bool lcMesh::FileLoad(lcMemFile& File)
 				File.ReadBuffer(FileName, Length);
 				FileName[Length] = 0;
 
-				Section.Texture = lcGetPiecesLibrary()->FindTexture(FileName);
+				Section.Texture = lcGetPiecesLibrary()->FindTexture(FileName, nullptr, false);
 			}
 			else
-				Section.Texture = NULL;
+				Section.Texture = nullptr;
 		}
 	}
 
-	File.ReadFloats((float*)mVertexData, 3 * mNumVertices + 5 * mNumTexturedVertices);
+	File.ReadBuffer(mVertexData, mNumVertices * sizeof(lcVertex) + mNumTexturedVertices * sizeof(lcVertexTextured));
 	if (mIndexType == GL_UNSIGNED_SHORT)
 		File.ReadU16((lcuint16*)mIndexData, mIndexDataSize / 2);
 	else
@@ -403,7 +444,7 @@ bool lcMesh::FileSave(lcMemFile& File)
 		}
 	}
 
-	File.WriteFloats((float*)mVertexData, 3 * mNumVertices + 5 * mNumTexturedVertices);
+	File.WriteBuffer(mVertexData, mNumVertices * sizeof(lcVertex) + mNumTexturedVertices * sizeof(lcVertexTextured));
 	if (mIndexType == GL_UNSIGNED_SHORT)
 		File.WriteU16((lcuint16*)mIndexData, mIndexDataSize / 2);
 	else
