@@ -19,7 +19,9 @@
 #include "lc_qminifigdialog.h"
 #include "lc_qgroupdialog.h"
 #include "lc_qeditgroupsdialog.h"
+#include "lc_selectbycolordialog.h"
 #include "lc_qutils.h"
+#include "lc_lxf.h"
 
 void lcModelProperties::LoadDefaults()
 {
@@ -32,9 +34,6 @@ void lcModelProperties::LoadDefaults()
 	mBackgroundImage = lcGetProfileString(LC_PROFILE_DEFAULT_BACKGROUND_TEXTURE);
 	mBackgroundImageTile = lcGetProfileInt(LC_PROFILE_DEFAULT_BACKGROUND_TILE);
 
-	mFogEnabled = lcGetProfileInt(LC_PROFILE_DEFAULT_FOG_ENABLED);
-	mFogDensity = lcGetProfileFloat(LC_PROFILE_DEFAULT_FOG_DENSITY);
-	mFogColor = lcVector3FromColor(lcGetProfileInt(LC_PROFILE_DEFAULT_FOG_COLOR));
 	mAmbientColor = lcVector3FromColor(lcGetProfileInt(LC_PROFILE_DEFAULT_AMBIENT_COLOR));
 }
 
@@ -47,9 +46,6 @@ void lcModelProperties::SaveDefaults()
 	lcSetProfileString(LC_PROFILE_DEFAULT_BACKGROUND_TEXTURE, mBackgroundImage);
 	lcSetProfileInt(LC_PROFILE_DEFAULT_BACKGROUND_TILE, mBackgroundImageTile);
 
-	lcSetProfileInt(LC_PROFILE_DEFAULT_FOG_ENABLED, mFogEnabled);
-	lcSetProfileFloat(LC_PROFILE_DEFAULT_FOG_DENSITY, mFogDensity);
-	lcSetProfileInt(LC_PROFILE_DEFAULT_FOG_COLOR, lcColorFromVector3(mFogColor));
 	lcSetProfileInt(LC_PROFILE_DEFAULT_AMBIENT_COLOR, lcColorFromVector3(mAmbientColor));
 }
 
@@ -96,9 +92,6 @@ void lcModelProperties::SaveLDraw(QTextStream& Stream) const
 		break;
 	}
 
-//	bool mFogEnabled;
-//	float mFogDensity;
-//	lcVector3 mFogColor;
 //	lcVector3 mAmbientColor;
 }
 
@@ -143,7 +136,10 @@ void lcModelProperties::ParseLDrawLine(QTextStream& Stream)
 			}
 
 			if (Token == QLatin1String("NAME"))
+			{
 				mBackgroundImage = Stream.readLine().trimmed();
+				mBackgroundType = LC_BACKGROUND_IMAGE;
+			}
 		}
 	}
 }
@@ -155,8 +151,8 @@ lcModel::lcModel(const QString& Name)
 
 	mActive = false;
 	mCurrentStep = 1;
-	mBackgroundTexture = NULL;
-	mPieceInfo = NULL;
+	mBackgroundTexture = nullptr;
+	mPieceInfo = nullptr;
 }
 
 lcModel::~lcModel()
@@ -164,7 +160,7 @@ lcModel::~lcModel()
 	if (mPieceInfo)
 	{
 		if (gMainWindow && gMainWindow->GetCurrentPieceInfo() == mPieceInfo)
-			gMainWindow->SetCurrentPieceInfo(NULL);
+			gMainWindow->SetCurrentPieceInfo(nullptr);
 
 		if (mPieceInfo->GetModel() == this)
 			mPieceInfo->SetPlaceholder();
@@ -198,7 +194,7 @@ void lcModel::DeleteHistory()
 void lcModel::DeleteModel()
 {
 	lcReleaseTexture(mBackgroundTexture);
-	mBackgroundTexture = NULL;
+	mBackgroundTexture = nullptr;
 
 	if (gMainWindow)
 	{
@@ -229,7 +225,7 @@ void lcModel::CreatePieceInfo(Project* Project)
 {
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
 	mPieceInfo = Library->FindPiece(mProperties.mName.toUpper().toLatin1().constData(), Project, true, false);
-	mPieceInfo->SetModel(this, true);
+	mPieceInfo->SetModel(this, true, Project, true);
 	Library->LoadPieceInfo(mPieceInfo, true, true);
 }
 
@@ -238,7 +234,7 @@ void lcModel::UpdatePieceInfo(lcArray<lcModel*>& UpdatedModels)
 	if (UpdatedModels.FindIndex(this) != -1)
 		return;
 
-	mPieceInfo->SetModel(this, false);
+	mPieceInfo->SetModel(this, false, nullptr, false);
 	UpdatedModels.Add(this);
 
 	lcMesh* Mesh = mPieceInfo->GetMesh();
@@ -277,12 +273,10 @@ void lcModel::SaveLDraw(QTextStream& Stream, bool SelectedOnly) const
 
 	mProperties.SaveLDraw(Stream);
 
-	if (mCurrentStep != GetLastStep())
-		Stream << QLatin1String("0 !LEOCAD MODEL CURRENT_STEP") << mCurrentStep << LineEnding;
-
 	lcArray<lcGroup*> CurrentGroups;
 	lcStep Step = 1;
 	int CurrentLine = 0;
+	int AddedSteps = 0;
 
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
 	{
@@ -314,13 +308,18 @@ void lcModel::SaveLDraw(QTextStream& Stream, bool SelectedOnly) const
 			}
 
 			if (!Skip)
+			{
 				Stream << mFileLines[CurrentLine];
+				if (AddedSteps > 0)
+					AddedSteps--;
+			}
 			CurrentLine++;
 		}
 
 		while (Piece->GetStepShow() > Step)
 		{
 			Stream << QLatin1String("0 STEP\r\n");
+			AddedSteps++;
 			Step++;
 		}
 
@@ -400,7 +399,23 @@ void lcModel::SaveLDraw(QTextStream& Stream, bool SelectedOnly) const
 
 	while (CurrentLine < mFileLines.size())
 	{
-		Stream << mFileLines[CurrentLine];
+		QString Line = mFileLines[CurrentLine];
+		QTextStream LineStream(&Line, QIODevice::ReadOnly);
+
+		QString Token;
+		LineStream >> Token;
+		bool Skip = false;
+
+		if (Token == QLatin1String("0"))
+		{
+			LineStream >> Token;
+
+			if (Token == QLatin1String("STEP") && AddedSteps-- > 0)
+				Skip = true;
+		}
+
+		if (!Skip)
+			Stream << mFileLines[CurrentLine];
 		CurrentLine++;
 	}
 
@@ -465,9 +480,9 @@ void lcModel::SplitMPD(QIODevice& Device)
 
 void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 {
-	lcPiece* Piece = NULL;
-	lcCamera* Camera = NULL;
-	lcLight* Light = NULL;
+	lcPiece* Piece = nullptr;
+	lcCamera* Camera = nullptr;
+	lcLight* Light = nullptr;
 	lcArray<lcGroup*> CurrentGroups;
 	lcArray<lcPieceControlPoint> ControlPoints;
 	int CurrentStep = 1;
@@ -527,7 +542,7 @@ void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 			else if (Token == QLatin1String("PIECE"))
 			{
 				if (!Piece)
-					Piece = new lcPiece(NULL);
+					Piece = new lcPiece(nullptr);
 
 				Piece->ParseLDrawLine(LineStream);
 			}
@@ -540,7 +555,7 @@ void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 				{
 					Camera->CreateName(mCameras);
 					mCameras.Add(Camera);
-					Camera = NULL;
+					Camera = nullptr;
 				}
 			}
 			else if (Token == QLatin1String("LIGHT"))
@@ -558,7 +573,7 @@ void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 					if (!CurrentGroups.IsEmpty())
 						Group->mGroup = CurrentGroups[CurrentGroups.GetSize() - 1];
 					else
-						Group->mGroup = NULL;
+						Group->mGroup = nullptr;
 					CurrentGroups.Add(Group);
 				}
 				else if (Token == QLatin1String("END"))
@@ -620,7 +635,7 @@ void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 			else
 			{
 				if (!Piece)
-					Piece = new lcPiece(NULL);
+					Piece = new lcPiece(nullptr);
 
 				if (!CurrentGroups.IsEmpty())
 					Piece->SetGroup(CurrentGroups[CurrentGroups.GetSize() - 1]);
@@ -640,7 +655,7 @@ void lcModel::LoadLDraw(QIODevice& Device, Project* Project)
 				Piece->SetColorCode(ColorCode);
 				Piece->SetControlPoints(ControlPoints);
 				AddPiece(Piece);
-				Piece = NULL;
+				Piece = nullptr;
 			}
 		}
 		else
@@ -713,7 +728,7 @@ bool lcModel::LoadBinary(lcFile* file)
 	{
 		if (fv > 0.4f)
 		{
-			lcPiece* pPiece = new lcPiece(NULL);
+			lcPiece* pPiece = new lcPiece(nullptr);
 			pPiece->FileLoad(*file);
 			AddPiece(pPiece);
 		}
@@ -734,7 +749,7 @@ bool lcModel::LoadBinary(lcFile* file)
 			lcMatrix44 WorldMatrix = lcMul(lcMatrix44RotationZ(rot[2] * LC_DTOR), lcMul(lcMatrix44RotationY(rot[1] * LC_DTOR), lcMatrix44RotationX(rot[0] * LC_DTOR)));
 			WorldMatrix.SetTranslation(pos);
 
-			PieceInfo* pInfo = Library->FindPiece(name, NULL, true, false);
+			PieceInfo* pInfo = Library->FindPiece(name, nullptr, true, false);
 			lcPiece* pPiece = new lcPiece(pInfo);
 
 			pPiece->Initialize(WorldMatrix, step);
@@ -817,7 +832,7 @@ bool lcModel::LoadBinary(lcFile* file)
 			lcGroup* Group = mGroups[GroupIdx];
 
 			i = LC_POINTER_TO_INT(Group->mGroup);
-			Group->mGroup = NULL;
+			Group->mGroup = nullptr;
 
 			if (i > 0xFFFF || i == -1)
 				continue;
@@ -830,7 +845,7 @@ bool lcModel::LoadBinary(lcFile* file)
 			lcPiece* Piece = mPieces[PieceIdx];
 
 			i = LC_POINTER_TO_INT(Piece->GetGroup());
-			Piece->SetGroup(NULL);
+			Piece->SetGroup(nullptr);
 
 			if (i > 0xFFFF || i == -1)
 				continue;
@@ -868,20 +883,7 @@ bool lcModel::LoadBinary(lcFile* file)
 
 	if (fv >= 0.7f)
 	{
-		file->Seek(16, SEEK_CUR);
-
-		file->ReadU32(&rgb, 1);
-		mProperties.mFogColor[0] = (float)((unsigned char) (rgb))/255;
-		mProperties.mFogColor[1] = (float)((unsigned char) (((unsigned short) (rgb)) >> 8))/255;
-		mProperties.mFogColor[2] = (float)((unsigned char) ((rgb) >> 16))/255;
-
-		if (fv < 1.0f)
-		{
-			file->ReadU32(&rgb, 1);
-			mProperties.mFogDensity = (float)rgb/100;
-		}
-		else
-			file->ReadFloats(&mProperties.mFogDensity, 1);
+		file->Seek(24, SEEK_CUR);
 
 		if (fv < 1.3f)
 		{
@@ -943,6 +945,33 @@ bool lcModel::LoadBinary(lcFile* file)
 	return true;
 }
 
+bool lcModel::LoadLDD(const QString& FileData)
+{
+	lcArray<lcPiece*> Pieces;
+	lcArray<lcArray<lcPiece*>> Groups;
+	
+	if (!lcImportLDDFile(FileData, Pieces, Groups))
+		return false;
+
+	for (lcPiece* Piece : Pieces)
+		AddPiece(Piece);
+
+	for (const lcArray<lcPiece*>& Group : Groups)
+	{
+		lcGroup* NewGroup = AddGroup(tr("Group #"), nullptr);
+		for (lcPiece* Piece : Group)
+			Piece->SetGroup(NewGroup);
+	}
+
+	lcPiecesLibrary* Library = lcGetPiecesLibrary();
+	CalculateStep(mCurrentStep);
+	Library->WaitForLoadQueue();
+	Library->mBuffersDirty = true;
+	Library->UnloadUnusedParts();
+
+	return true;
+}
+
 void lcModel::Merge(lcModel* Other)
 {
 	for (int PieceIdx = 0; PieceIdx < Other->mPieces.GetSize(); PieceIdx++)
@@ -995,7 +1024,7 @@ void lcModel::Cut()
 		gMainWindow->UpdateTimeline(false, false);
 		gMainWindow->UpdateSelectedObjects(true);
 		gMainWindow->UpdateAllViews();
-		SaveCheckpoint("Cutting");
+		SaveCheckpoint(tr("Cutting"));
 	}
 }
 
@@ -1029,6 +1058,7 @@ void lcModel::Paste()
 		lcPiece* Piece = PastedPieces[PieceIdx];
 		lcStep Step = Piece->GetStepShow();
 
+		Piece->SetFileLine(-1);
 		if (Step > mCurrentStep)
 			Piece->SetStepShow(mCurrentStep);
 
@@ -1041,14 +1071,41 @@ void lcModel::Paste()
 	if (SelectedObjects.GetSize() == 1)
 		ClearSelectionAndSetFocus(SelectedObjects[0], LC_PIECE_SECTION_POSITION);
 	else
-		SetSelectionAndFocus(SelectedObjects, NULL, 0);
+		SetSelectionAndFocus(SelectedObjects, nullptr, 0);
 
 	CalculateStep(mCurrentStep);
 	gMainWindow->UpdateTimeline(false, false);
 	gMainWindow->UpdateAllViews();
 }
 
-void lcModel::GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface) const
+void lcModel::DuplicateSelectedPieces()
+{
+	lcArray<lcObject*> NewPieces;
+
+	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
+	{
+		lcPiece* Piece = mPieces[PieceIdx];
+
+		if (!Piece->IsSelected())
+			continue;
+
+		lcPiece* NewPiece = new lcPiece(*Piece);
+		NewPiece->UpdatePosition(mCurrentStep);
+		NewPieces.Add(NewPiece);
+
+		PieceIdx++;
+		InsertPiece(NewPiece, PieceIdx);
+	}
+
+	if (NewPieces.IsEmpty())
+		return;
+
+	gMainWindow->UpdateTimeline(false, false);
+	SetSelectionAndFocus(NewPieces, nullptr, 0);
+	SaveCheckpoint(tr("Duplicating Pieces"));
+}
+
+void lcModel::GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface, bool Highlight) const
 {
 	Scene.Begin(ViewCamera->mWorldView);
 
@@ -1059,7 +1116,7 @@ void lcModel::GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface)
 		lcPiece* Piece = mPieces[PieceIdx];
 
 		if (Piece->IsVisible(mCurrentStep))
-			Piece->AddRenderMeshes(Scene, DrawInterface);
+			Piece->AddRenderMeshes(Scene, DrawInterface, Highlight && Piece->GetStepShow() == mCurrentStep);
 	}
 
 	if (DrawInterface)
@@ -1069,7 +1126,7 @@ void lcModel::GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface)
 			lcCamera* Camera = mCameras[CameraIdx];
 
 			if (Camera != ViewCamera && Camera->IsVisible())
-				Scene.mInterfaceObjects.Add(Camera);
+				Scene.AddInterfaceObject(Camera);
 		}
 
 		for (int LightIdx = 0; LightIdx < mLights.GetSize(); LightIdx++)
@@ -1077,7 +1134,7 @@ void lcModel::GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface)
 			lcLight* Light = mLights[LightIdx];
 
 			if (Light->IsVisible())
-				Scene.mInterfaceObjects.Add(Light);
+				Scene.AddInterfaceObject(Light);
 		}
 	}
 
@@ -1090,16 +1147,8 @@ void lcModel::SubModelAddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMat
 	{
 		lcPiece* Piece = mPieces[PieceIdx];
 
-		if (Piece->GetStepHide() != LC_STEP_MAX)
-			continue;
-
-		int ColorIndex = Piece->mColorIndex;
-
-		if (ColorIndex == gDefaultColor)
-			ColorIndex = DefaultColorIndex;
-
-		PieceInfo* Info = Piece->mPieceInfo;
-		Info->AddRenderMeshes(Scene, lcMul(Piece->mModelWorld, WorldMatrix), ColorIndex, Focused, Selected);
+		if (Piece->GetStepHide() == LC_STEP_MAX)
+			Piece->SubModelAddRenderMeshes(Scene, WorldMatrix, DefaultColorIndex, Focused, Selected);
 	}
 }
 
@@ -1116,9 +1165,6 @@ void lcModel::DrawBackground(lcGLWidget* Widget)
 
 	glDepthMask(GL_FALSE);
 	glDisable(GL_DEPTH_TEST);
-#ifndef LC_OPENGLES
-	glDisable(GL_LIGHTING);
-#endif
 
 	float ViewWidth = (float)Widget->mWidth;
 	float ViewHeight = (float)Widget->mHeight;
@@ -1130,9 +1176,7 @@ void lcModel::DrawBackground(lcGLWidget* Widget)
 
 	if (mProperties.mBackgroundType == LC_BACKGROUND_GRADIENT)
 	{
-#ifndef LC_OPENGLES
-		glShadeModel(GL_SMOOTH);
-#endif
+		Context->SetSmoothShading(true);
 
 		const lcVector3& Color1 = mProperties.mBackgroundGradientColor1;
 		const lcVector3& Color2 = mProperties.mBackgroundGradientColor2;
@@ -1145,24 +1189,17 @@ void lcModel::DrawBackground(lcGLWidget* Widget)
 			ViewWidth, 0.0f,       Color2[0], Color2[1], Color2[2], 1.0f
 		};
 
-		Context->SetProgram(LC_PROGRAM_VERTEX_COLOR);
+		Context->SetMaterial(LC_MATERIAL_UNLIT_VERTEX_COLOR);
 		Context->SetVertexBufferPointer(Verts);
-		Context->SetVertexFormat(0, 2, 0, 4);
+		Context->SetVertexFormat(0, 2, 0, 0, 4, false);
 
 		Context->DrawPrimitives(GL_TRIANGLE_FAN, 0, 4);
 
-		Context->ClearVertexBuffer(); // context remove
-
-#ifndef LC_OPENGLES
-		glShadeModel(GL_FLAT);
-#endif
+		Context->SetSmoothShading(false);
 	}
 	else if (mProperties.mBackgroundType == LC_BACKGROUND_IMAGE)
 	{
-		glEnable(GL_TEXTURE_2D);
-
-		Context->SetTextureMode(LC_TEXTURE_REPLACE);
-		glBindTexture(GL_TEXTURE_2D, mBackgroundTexture->mTexture);
+		Context->BindTexture(mBackgroundTexture->mTexture);
 
 		float TileWidth = 1.0f, TileHeight = 1.0f;
 
@@ -1180,22 +1217,19 @@ void lcModel::DrawBackground(lcGLWidget* Widget)
 			0.0f,      0.0f,       0.0f,      TileHeight
 		};
 
-		Context->SetProgram(LC_PROGRAM_TEXTURE);
+		Context->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
+		Context->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_DECAL);
 		Context->SetVertexBufferPointer(Verts);
-		Context->SetVertexFormat(0, 2, 2, 0);
+		Context->SetVertexFormat(0, 2, 0, 2, 0, false);
 
 		Context->DrawPrimitives(GL_TRIANGLE_FAN, 0, 4);
-
-		Context->ClearVertexBuffer(); // context remove
-
-		glDisable(GL_TEXTURE_2D);
 	}
 
 	glEnable(GL_DEPTH_TEST);
 	glDepthMask(GL_TRUE);
 }
 
-void lcModel::SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Zoom, int Width, int Height, lcStep Start, lcStep End)
+void lcModel::SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Zoom, bool Highlight, int Width, int Height, lcStep Start, lcStep End)
 {
 	View* ActiveView = gMainWindow->GetActiveView();
 	ActiveView->MakeCurrent();
@@ -1214,6 +1248,7 @@ void lcModel::SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Z
 		ZoomExtents(Camera, (float)Width / (float)Height);
 
 	View View(this);
+	View.SetHighlight(Highlight);
 	View.SetCamera(Camera, false);
 	View.mWidth = Width;
 	View.mHeight = Height;
@@ -1236,6 +1271,7 @@ void lcModel::SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Z
 	}
 
 	Context->EndRenderToTexture();
+	Context->ClearResources();
 
 	SetTemporaryStep(CurrentStep);
 
@@ -1246,7 +1282,7 @@ void lcModel::SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Z
 void lcModel::UpdateBackgroundTexture()
 {
 	lcReleaseTexture(mBackgroundTexture);
-	mBackgroundTexture = NULL;
+	mBackgroundTexture = nullptr;
 
 	if (mProperties.mBackgroundType == LC_BACKGROUND_IMAGE)
 	{
@@ -1556,7 +1592,7 @@ lcGroup* lcModel::GetGroup(const QString& Name, bool CreateIfMissing)
 		return Group;
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void lcModel::RemoveGroup(lcGroup* Group)
@@ -1623,7 +1659,7 @@ void lcModel::UngroupSelection()
 		lcGroup* Group = Piece->GetGroup();
 
 		if (SelectedGroups.FindIndex(Group) != -1)
-			Piece->SetGroup(NULL);
+			Piece->SetGroup(nullptr);
 	}
 
 	for (int GroupIdx = 0; GroupIdx < mGroups.GetSize(); GroupIdx++)
@@ -1631,7 +1667,7 @@ void lcModel::UngroupSelection()
 		lcGroup* Group = mGroups[GroupIdx];
 
 		if (SelectedGroups.FindIndex(Group->mGroup) != -1)
-			Group->mGroup = NULL;
+			Group->mGroup = nullptr;
 	}
 
 	SelectedGroups.DeleteAll();
@@ -1642,7 +1678,7 @@ void lcModel::UngroupSelection()
 
 void lcModel::AddSelectedPiecesToGroup()
 {
-	lcGroup* Group = NULL;
+	lcGroup* Group = nullptr;
 
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
 	{
@@ -1682,7 +1718,7 @@ void lcModel::RemoveFocusPieceFromGroup()
 
 		if (Piece->IsFocused())
 		{
-			Piece->SetGroup(NULL);
+			Piece->SetGroup(nullptr);
 			break;
 		}
 	}
@@ -1852,15 +1888,9 @@ lcVector3 lcModel::SnapPosition(const lcVector3& Distance) const
 		float Leftover = NewDistance[0] - (SnapXY * i);
 
 		if (Leftover > SnapXY / 2)
-		{
-			Leftover -= SnapXY;
 			i++;
-		}
 		else if (Leftover < -SnapXY / 2)
-		{
-			Leftover += SnapXY;
 			i--;
-		}
 
 		NewDistance[0] = SnapXY * i;
 
@@ -1868,15 +1898,9 @@ lcVector3 lcModel::SnapPosition(const lcVector3& Distance) const
 		Leftover = NewDistance[1] - (SnapXY * i);
 
 		if (Leftover > SnapXY / 2)
-		{
-			Leftover -= SnapXY;
 			i++;
-		}
 		else if (Leftover < -SnapXY / 2)
-		{
-			Leftover += SnapXY;
 			i--;
-		}
 
 		NewDistance[1] = SnapXY * i;
 	}
@@ -1888,15 +1912,9 @@ lcVector3 lcModel::SnapPosition(const lcVector3& Distance) const
 		float Leftover = NewDistance[2] - (SnapZ * i);
 
 		if (Leftover > SnapZ / 2)
-		{
-			Leftover -= SnapZ;
 			i++;
-		}
 		else if (Leftover < -SnapZ / 2)
-		{
-			Leftover += SnapZ;
 			i--;
-		}
 
 		NewDistance[2] = SnapZ * i;
 	}
@@ -1942,7 +1960,7 @@ void lcModel::AddPiece()
 	if (!CurPiece)
 		return;
 
-	lcPiece* Last = mPieces.IsEmpty() ? NULL : mPieces[mPieces.GetSize() - 1];
+	lcPiece* Last = mPieces.IsEmpty() ? nullptr : mPieces[mPieces.GetSize() - 1];
 
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
 	{
@@ -1979,7 +1997,7 @@ void lcModel::AddPiece()
 	gMainWindow->UpdateTimeline(false, false);
 	ClearSelectionAndSetFocus(Piece, LC_PIECE_SECTION_POSITION);
 
-	SaveCheckpoint("Adding Piece");
+	SaveCheckpoint(tr("Adding Piece"));
 }
 
 void lcModel::AddPiece(lcPiece* Piece)
@@ -2021,7 +2039,7 @@ void lcModel::DeleteAllCameras()
 	gMainWindow->UpdateCameraMenu();
 	gMainWindow->UpdateSelectedObjects(true);
 	gMainWindow->UpdateAllViews();
-	SaveCheckpoint("Reseting Cameras");
+	SaveCheckpoint(tr("Reseting Cameras"));
 }
 
 void lcModel::DeleteSelectedObjects()
@@ -2031,7 +2049,7 @@ void lcModel::DeleteSelectedObjects()
 		gMainWindow->UpdateTimeline(false, false);
 		gMainWindow->UpdateSelectedObjects(true);
 		gMainWindow->UpdateAllViews();
-		SaveCheckpoint("Deleting");
+		SaveCheckpoint(tr("Deleting"));
 	}
 }
 
@@ -2062,7 +2080,7 @@ void lcModel::InsertControlPoint()
 
 	if (Piece->InsertControlPoint(Start, End))
 	{
-		SaveCheckpoint("Modifying");
+		SaveCheckpoint(tr("Modifying"));
 		gMainWindow->UpdateSelectedObjects(true);
 		gMainWindow->UpdateAllViews();
 	}
@@ -2079,7 +2097,7 @@ void lcModel::RemoveFocusedControlPoint()
 
 	if (Piece->RemoveFocusedControlPoint())
 	{
-		SaveCheckpoint("Modifying");
+		SaveCheckpoint(tr("Modifying"));
 		gMainWindow->UpdateSelectedObjects(true);
 		gMainWindow->UpdateAllViews();
 	}
@@ -2121,7 +2139,7 @@ void lcModel::ShowSelectedPiecesEarlier()
 		AddPiece(Piece);
 	}
 
-	SaveCheckpoint("Modifying");
+	SaveCheckpoint(tr("Modifying"));
 	gMainWindow->UpdateTimeline(false, false);
 	gMainWindow->UpdateSelectedObjects(true);
 	gMainWindow->UpdateAllViews();
@@ -2166,7 +2184,7 @@ void lcModel::ShowSelectedPiecesLater()
 		AddPiece(Piece);
 	}
 
-	SaveCheckpoint("Modifying");
+	SaveCheckpoint(tr("Modifying"));
 	gMainWindow->UpdateTimeline(false, false);
 	gMainWindow->UpdateSelectedObjects(true);
 	gMainWindow->UpdateAllViews();
@@ -2201,7 +2219,7 @@ void lcModel::SetPieceSteps(const QList<QPair<lcPiece*, lcStep>>& PieceSteps)
 
 	if (Modified)
 	{
-		SaveCheckpoint("Modifying");
+		SaveCheckpoint(tr("Modifying"));
 		gMainWindow->UpdateAllViews();
 		gMainWindow->UpdateTimeline(false, false);
 		gMainWindow->UpdateSelectedObjects(true);
@@ -2214,7 +2232,8 @@ void lcModel::MoveSelectionToModel(lcModel* Model)
 		return;
 
 	lcArray<lcPiece*> Pieces;
-	lcPiece* ModelPiece = NULL;
+	lcPiece* ModelPiece = nullptr;
+	lcStep FirstStep = LC_STEP_MAX;
 
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); )
 	{
@@ -2223,8 +2242,9 @@ void lcModel::MoveSelectionToModel(lcModel* Model)
 		if (Piece->IsSelected())
 		{
 			mPieces.RemoveIndex(PieceIdx);
-			Piece->SetGroup(NULL); // todo: copy groups
+			Piece->SetGroup(nullptr); // todo: copy groups
 			Pieces.Add(Piece);
+			FirstStep = qMin(FirstStep, Piece->GetStepShow());
 
 			if (!ModelPiece)
 			{
@@ -2240,13 +2260,19 @@ void lcModel::MoveSelectionToModel(lcModel* Model)
 	}
 
 	for (int PieceIdx = 0; PieceIdx < Pieces.GetSize(); PieceIdx++)
-		Model->AddPiece(Pieces[PieceIdx]);
+	{
+		lcPiece* Piece = Pieces[PieceIdx];
+		Piece->SetFileLine(-1);
+		Piece->SetStepShow(Piece->GetStepShow() - FirstStep + 1);
+		Model->AddPiece(Piece);
+	}
 
 	lcArray<lcModel*> UpdatedModels;
 	Model->UpdatePieceInfo(UpdatedModels);
-	ModelPiece->UpdatePosition(mCurrentStep);
+	if (ModelPiece)
+		ModelPiece->UpdatePosition(mCurrentStep);
 
-	SaveCheckpoint("New Model");
+	SaveCheckpoint(tr("New Model"));
 	gMainWindow->UpdateTimeline(false, false);
 	ClearSelectionAndSetFocus(ModelPiece, LC_PIECE_SECTION_POSITION);
 }
@@ -2296,9 +2322,9 @@ void lcModel::InlineSelectedModels()
 		return;
 	}
 
-	SaveCheckpoint("Inlining");
+	SaveCheckpoint(tr("Inlining"));
 	gMainWindow->UpdateTimeline(false, false);
-	SetSelectionAndFocus(NewPieces, NULL, 0);
+	SetSelectionAndFocus(NewPieces, nullptr, 0);
 }
 
 bool lcModel::RemoveSelectedObjects()
@@ -2443,7 +2469,7 @@ void lcModel::MoveSelectedObjects(const lcVector3& PieceDistance, const lcVector
 	{
 		gMainWindow->UpdateAllViews();
 		if (Checkpoint)
-			SaveCheckpoint("Moving");
+			SaveCheckpoint(tr("Moving"));
 		gMainWindow->UpdateSelectedObjects(false);
 	}
 }
@@ -2509,7 +2535,7 @@ void lcModel::RotateSelectedPieces(const lcVector3& Angles, bool Relative, bool
 	{
 		gMainWindow->UpdateAllViews();
 		if (Checkpoint)
-			SaveCheckpoint("Rotating");
+			SaveCheckpoint(tr("Rotating"));
 		gMainWindow->UpdateSelectedObjects(false);
 	}
 }
@@ -2535,7 +2561,7 @@ void lcModel::ScaleSelectedPieces(const float Scale, bool Update, bool Checkpoin
 		{
 			gMainWindow->UpdateAllViews();
 			if (Checkpoint)
-				SaveCheckpoint("Scaling");
+				SaveCheckpoint(tr("Scaling"));
 			gMainWindow->UpdateSelectedObjects(false);
 		}
 	}
@@ -2792,7 +2818,7 @@ lcObject* lcModel::GetFocusObject() const
 			return Light;
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 lcModel* lcModel::GetFirstSelectedSubmodel() const
@@ -2805,7 +2831,7 @@ lcModel* lcModel::GetFirstSelectedSubmodel() const
 			return Piece->mPieceInfo->GetModel();
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void lcModel::GetSubModels(lcArray<lcModel*> SubModels) const
@@ -2904,7 +2930,7 @@ bool lcModel::GetMoveRotateTransform(lcVector3& Center, lcMatrix33& RelativeRota
 bool lcModel::GetPieceFocusOrSelectionCenter(lcVector3& Center) const
 {
 	lcVector3 Min(FLT_MAX, FLT_MAX, FLT_MAX), Max(-FLT_MAX, -FLT_MAX, -FLT_MAX);
-	lcPiece* Selected = NULL;
+	lcPiece* Selected = nullptr;
 	int NumSelected = 0;
 
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
@@ -2971,7 +2997,7 @@ bool lcModel::GetFocusPosition(lcVector3& Position) const
 bool lcModel::GetSelectionCenter(lcVector3& Center) const
 {
 	lcVector3 Min(FLT_MAX, FLT_MAX, FLT_MAX), Max(-FLT_MAX, -FLT_MAX, -FLT_MAX);
-	lcPiece* SelectedPiece = NULL;
+	lcPiece* SelectedPiece = nullptr;
 	bool SinglePieceSelected = true;
 	bool Selected = false;
 
@@ -3027,21 +3053,22 @@ bool lcModel::GetSelectionCenter(lcVector3& Center) const
 
 bool lcModel::GetPiecesBoundingBox(lcVector3& Min, lcVector3& Max) const
 {
+	bool Valid = false;
 	Min = lcVector3(FLT_MAX, FLT_MAX, FLT_MAX);
 	Max = lcVector3(-FLT_MAX, -FLT_MAX, -FLT_MAX);
 
-	if (mPieces.IsEmpty())
-		return false;
-
 	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
 	{
 		lcPiece* Piece = mPieces[PieceIdx];
 
 		if (Piece->IsVisible(mCurrentStep))
+		{
 			Piece->CompareBoundingBox(Min, Max);
+			Valid = true;
+		}
 	}
 
-	return true;
+	return Valid;
 }
 
 void lcModel::GetPartsList(int DefaultColorIndex, lcPartsList& PartsList) const
@@ -3095,13 +3122,13 @@ void lcModel::GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex
 void lcModel::GetSelectionInformation(int* Flags, lcArray<lcObject*>& Selection, lcObject** Focus) const
 {
 	*Flags = 0;
-	*Focus = NULL;
+	*Focus = nullptr;
 
 	if (mPieces.IsEmpty())
 		*Flags |= LC_SEL_NO_PIECES;
 	else
 	{
-		lcGroup* Group = NULL;
+		lcGroup* Group = nullptr;
 		bool First = true;
 
 		for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
@@ -3136,7 +3163,7 @@ void lcModel::GetSelectionInformation(int* Flags, lcArray<lcObject*>& Selection,
 						*Flags |= LC_SEL_CAN_REMOVE_CONTROL_POINT;
 				}
 
-				if (Piece->GetGroup() != NULL)
+				if (Piece->GetGroup() != nullptr)
 				{
 					*Flags |= LC_SEL_GROUPED;
 					if (Piece->IsFocused())
@@ -3153,7 +3180,7 @@ void lcModel::GetSelectionInformation(int* Flags, lcArray<lcObject*>& Selection,
 					if (Group != Piece->GetGroup())
 						*Flags |= LC_SEL_CAN_GROUP;
 					else
-						if (Group == NULL)
+						if (Group == nullptr)
 							*Flags |= LC_SEL_CAN_GROUP;
 				}
 			}
@@ -3423,7 +3450,7 @@ void lcModel::FindPiece(bool FindFirst, bool SearchForward)
 	}
 
 	int CurrentIdx = StartIdx;
-	lcObject* Focus = NULL;
+	lcObject* Focus = nullptr;
 	const lcSearchOptions& SearchOptions = gMainWindow->mSearchOptions;
 
 	for (;;)
@@ -3470,7 +3497,7 @@ void lcModel::UndoAction()
 	LoadCheckPoint(mUndoHistory[0]);
 
 	gMainWindow->UpdateModified(IsModified());
-	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : NULL, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : NULL);
+	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : nullptr, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : nullptr);
 }
 
 void lcModel::RedoAction()
@@ -3485,7 +3512,7 @@ void lcModel::RedoAction()
 	LoadCheckPoint(Redo);
 
 	gMainWindow->UpdateModified(IsModified());
-	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : NULL, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : NULL);
+	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : nullptr, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : nullptr);
 }
 
 void lcModel::BeginMouseTool()
@@ -3854,7 +3881,41 @@ void lcModel::ShowSelectByNameDialog()
 	if (Dialog.exec() != QDialog::Accepted)
 		return;
 
-	SetSelectionAndFocus(Dialog.mObjects, NULL, 0);
+	SetSelectionAndFocus(Dialog.mObjects, nullptr, 0);
+}
+
+void lcModel::ShowSelectByColorDialog()
+{
+	if (mPieces.IsEmpty())
+	{
+		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Nothing to select."));
+		return;
+	}
+
+	int ColorIndex = gMainWindow->mColorIndex;
+
+	lcObject* Focus = GetFocusObject();
+
+	if (Focus && Focus->IsPiece())
+		ColorIndex = ((lcPiece*)Focus)->mColorIndex;
+
+	lcSelectByColorDialog Dialog(gMainWindow, ColorIndex);
+
+	if (Dialog.exec() != QDialog::Accepted)
+		return;
+
+	ColorIndex = Dialog.mColorIndex;
+	lcArray<lcObject*> Selection;
+
+	for (int PieceIdx = 0; PieceIdx < mPieces.GetSize(); PieceIdx++)
+	{
+		lcPiece* Piece = mPieces[PieceIdx];
+
+		if (Piece->IsVisible(mCurrentStep) && Piece->mColorIndex == ColorIndex)
+			Selection.Add(Piece);
+	}
+
+	SetSelectionAndFocus(Selection, nullptr, 0);
 }
 
 void lcModel::ShowArrayDialog()
@@ -3941,13 +4002,13 @@ void lcModel::ShowMinifigDialog()
 
 	gMainWindow->GetActiveView()->MakeCurrent();
 
-	lcGroup* Group = AddGroup(tr("Minifig #"), NULL);
+	lcGroup* Group = AddGroup(tr("Minifig #"), nullptr);
 	lcArray<lcObject*> Pieces(LC_MFW_NUMITEMS);
 	lcMinifig& Minifig = Dialog.mMinifigWidget->mMinifig;
 
 	for (int PartIdx = 0; PartIdx < LC_MFW_NUMITEMS; PartIdx++)
 	{
-		if (Minifig.Parts[PartIdx] == NULL)
+		if (Minifig.Parts[PartIdx] == nullptr)
 			continue;
 
 		lcPiece* Piece = new lcPiece(Minifig.Parts[PartIdx]);
@@ -3961,7 +4022,7 @@ void lcModel::ShowMinifigDialog()
 		Pieces.Add(Piece);
 	}
 
-	SetSelectionAndFocus(Pieces, NULL, 0);
+	SetSelectionAndFocus(Pieces, nullptr, 0);
 	gMainWindow->UpdateTimeline(false, false);
 	SaveCheckpoint(tr("Minifig"));
 }
@@ -3969,7 +4030,7 @@ void lcModel::ShowMinifigDialog()
 void lcModel::UpdateInterface()
 {
 	gMainWindow->UpdateTimeline(true, false);
-	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : NULL, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : NULL);
+	gMainWindow->UpdateUndoRedo(mUndoHistory.GetSize() > 1 ? mUndoHistory[0]->Description : nullptr, !mRedoHistory.IsEmpty() ? mRedoHistory[0]->Description : nullptr);
 	gMainWindow->UpdatePaste(!g_App->mClipboard.isEmpty());
 	gMainWindow->UpdateCategories();
 	gMainWindow->UpdateTitle();
