@@ -8,6 +8,7 @@
 #include "project.h"
 #include "pieceinf.h"
 #include "view.h"
+#include "lc_glextensions.h"
 
  Q_DECLARE_METATYPE(QList<int>)
 
@@ -99,6 +100,7 @@ lcPartSelectionListModel::lcPartSelectionListModel(QObject* Parent)
 	mListView = (lcPartSelectionListView*)Parent;
 	mIconSize = 0;
 	mShowPartNames = lcGetProfileInt(LC_PROFILE_PARTS_LIST_NAMES);
+	mListMode = lcGetProfileInt(LC_PROFILE_PARTS_LIST_LISTMODE);
 
 	int ColorCode = lcGetProfileInt(LC_PROFILE_PARTS_LIST_COLOR);
 	if (ColorCode == -1)
@@ -162,6 +164,14 @@ void lcPartSelectionListModel::ToggleColorLocked()
 	lcSetProfileInt(LC_PROFILE_PARTS_LIST_COLOR, mColorLocked ? lcGetColorCode(mColorIndex) : -1);
 }
 
+void lcPartSelectionListModel::ToggleListMode()
+{
+	mListMode = !mListMode;
+
+	mListView->UpdateViewMode();
+	lcSetProfileInt(LC_PROFILE_PARTS_LIST_LISTMODE, mListMode);
+}
+
 void lcPartSelectionListModel::SetCategory(int CategoryIndex)
 {
 	ClearRequests();
@@ -219,8 +229,8 @@ void lcPartSelectionListModel::SetCurrentModelCategory()
 	lcPartsList PartsList;
 	CurrentModel->GetPartsList(gDefaultColor, PartsList);
 
-	for (lcPartsList::const_iterator PartIt = PartsList.constBegin(); PartIt != PartsList.constEnd(); PartIt++)
-		mParts.append(QPair<PieceInfo*, QPixmap>((PieceInfo*)PartIt.key(), QPixmap()));
+	for (const auto& PartIt : PartsList)
+		mParts.append(QPair<PieceInfo*, QPixmap>((PieceInfo*)PartIt.first, QPixmap()));
 
 	endResetModel();
 }
@@ -243,11 +253,8 @@ QVariant lcPartSelectionListModel::data(const QModelIndex& Index, int Role) cons
 		switch (Role)
 		{
 		case Qt::DisplayRole:
-			if (!mIconSize || mShowPartNames)
-			{
-				PieceInfo* Info = mParts[InfoIndex].first;
+			if (!mIconSize || mShowPartNames || mListMode)
 				return QVariant(QString::fromLatin1(Info->m_strDescription));
-			}
 			break;
 
 		case Qt::ToolTipRole:
@@ -334,7 +341,6 @@ void lcPartSelectionListModel::DrawPreview(int InfoIndex)
 
 	Context->SetDefaultState();
 	Context->SetProjectionMatrix(ProjectionMatrix);
-	Context->SetProgram(LC_PROGRAM_SIMPLE);
 
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
 	PieceInfo* Info = mParts[InfoIndex].first;
@@ -348,21 +354,18 @@ void lcPartSelectionListModel::DrawPreview(int InfoIndex)
 	lcScene Scene;
 	Scene.Begin(ViewMatrix);
 
-	Info->AddRenderMeshes(Scene, lcMatrix44Identity(), mColorIndex, false, false);
+	Info->AddRenderMeshes(Scene, lcMatrix44Identity(), mColorIndex, false, false, false);
 
 	Scene.End();
 
-	Context->SetViewMatrix(ViewMatrix);
-	Context->DrawOpaqueMeshes(Scene.mOpaqueMeshes);
-	Context->DrawTranslucentMeshes(Scene.mTranslucentMeshes);
-
-	Context->UnbindMesh(); // context remove
-		
-	Library->ReleasePieceInfo(Info);
+	Scene.Draw(Context);
 
 	mParts[InfoIndex].second = QPixmap::fromImage(Context->GetRenderToTextureImage(Width, Height)).scaled(mIconSize, mIconSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
 
+	Library->ReleasePieceInfo(Info);
+
 	Context->EndRenderToTexture();
+	Context->ClearResources();
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 2, 0))
 	emit dataChanged(index(InfoIndex, 0), index(InfoIndex, 0), QVector<int>() << Qt::DecorationRole);
@@ -422,36 +425,39 @@ void lcPartSelectionListView::CustomContextMenuRequested(QPoint Pos)
 {
 	QMenu* Menu = new QMenu(this);
 
-	QActionGroup* IconGroup = new QActionGroup(Menu);
+	if (gSupportsFramebufferObjectARB || gSupportsFramebufferObjectEXT)
+	{
+		QActionGroup* IconGroup = new QActionGroup(Menu);
 
-	QAction* NoIcons = Menu->addAction(tr("No Icons"), this, SLOT(SetNoIcons()));
-	NoIcons->setCheckable(true);
-	NoIcons->setChecked(mListModel->GetIconSize() == 0);
-	IconGroup->addAction(NoIcons);
+		QAction* NoIcons = Menu->addAction(tr("No Icons"), this, SLOT(SetNoIcons()));
+		NoIcons->setCheckable(true);
+		NoIcons->setChecked(mListModel->GetIconSize() == 0);
+		IconGroup->addAction(NoIcons);
 
-	QAction* SmallIcons = Menu->addAction(tr("Small Icons"), this, SLOT(SetSmallIcons()));
-	SmallIcons->setCheckable(true);
-	SmallIcons->setChecked(mListModel->GetIconSize() == 32);
-	IconGroup->addAction(SmallIcons);
+		QAction* SmallIcons = Menu->addAction(tr("Small Icons"), this, SLOT(SetSmallIcons()));
+		SmallIcons->setCheckable(true);
+		SmallIcons->setChecked(mListModel->GetIconSize() == 32);
+		IconGroup->addAction(SmallIcons);
 
-	QAction* MediumIcons = Menu->addAction(tr("Medium Icons"), this, SLOT(SetMediumIcons()));
-	MediumIcons->setCheckable(true);
-	MediumIcons->setChecked(mListModel->GetIconSize() == 64);
-	IconGroup->addAction(MediumIcons);
+		QAction* MediumIcons = Menu->addAction(tr("Medium Icons"), this, SLOT(SetMediumIcons()));
+		MediumIcons->setCheckable(true);
+		MediumIcons->setChecked(mListModel->GetIconSize() == 64);
+		IconGroup->addAction(MediumIcons);
 
-	QAction* LargeIcons = Menu->addAction(tr("Large Icons"), this, SLOT(SetLargeIcons()));
-	LargeIcons->setCheckable(true);
-	LargeIcons->setChecked(mListModel->GetIconSize() == 96);
-	IconGroup->addAction(LargeIcons);
+		QAction* LargeIcons = Menu->addAction(tr("Large Icons"), this, SLOT(SetLargeIcons()));
+		LargeIcons->setCheckable(true);
+		LargeIcons->setChecked(mListModel->GetIconSize() == 96);
+		IconGroup->addAction(LargeIcons);
 
-	QAction* ExtraLargeIcons = Menu->addAction(tr("Extra Large Icons"), this, SLOT(SetExtraLargeIcons()));
-	ExtraLargeIcons->setCheckable(true);
-	ExtraLargeIcons->setChecked(mListModel->GetIconSize() == 192);
-	IconGroup->addAction(ExtraLargeIcons);
+		QAction* ExtraLargeIcons = Menu->addAction(tr("Extra Large Icons"), this, SLOT(SetExtraLargeIcons()));
+		ExtraLargeIcons->setCheckable(true);
+		ExtraLargeIcons->setChecked(mListModel->GetIconSize() == 192);
+		IconGroup->addAction(ExtraLargeIcons);
 
-	Menu->addSeparator();
+		Menu->addSeparator();
+	}
 
-	if (mListModel->GetIconSize() != 0)
+	if (mListModel->GetIconSize() != 0 && !mListModel->IsListMode())
 	{
 		QAction* PartNames = Menu->addAction(tr("Show Part Names"), this, SLOT(TogglePartNames()));
 		PartNames->setCheckable(true);
@@ -462,9 +468,16 @@ void lcPartSelectionListView::CustomContextMenuRequested(QPoint Pos)
 	DecoratedParts->setCheckable(true);
 	DecoratedParts->setChecked(mFilterModel->GetShowDecoratedParts());
 
-	QAction* FixedColor = Menu->addAction(tr("Lock Preview Color"), this, SLOT(ToggleFixedColor()));
-	FixedColor->setCheckable(true);
-	FixedColor->setChecked(mListModel->IsColorLocked());
+	if (mListModel->GetIconSize() != 0)
+	{
+		QAction* ListMode = Menu->addAction(tr("List Mode"), this, SLOT(ToggleListMode()));
+		ListMode->setCheckable(true);
+		ListMode->setChecked(mListModel->IsListMode());
+
+		QAction* FixedColor = Menu->addAction(tr("Lock Preview Color"), this, SLOT(ToggleFixedColor()));
+		FixedColor->setCheckable(true);
+		FixedColor->setChecked(mListModel->IsColorLocked());
+	}
 
 	Menu->popup(viewport()->mapToGlobal(Pos));
 }
@@ -508,17 +521,28 @@ void lcPartSelectionListView::ToggleDecoratedParts()
 	lcSetProfileInt(LC_PROFILE_PARTS_LIST_DECORATED, Show);
 }
 
+void lcPartSelectionListView::ToggleListMode()
+{
+	mListModel->ToggleListMode();
+}
+
 void lcPartSelectionListView::ToggleFixedColor()
 {
 	mListModel->ToggleColorLocked();
 }
 
+void lcPartSelectionListView::UpdateViewMode()
+{
+	setViewMode(mListModel->GetIconSize() && !mListModel->IsListMode() ? QListView::IconMode : QListView::ListMode);
+	setWordWrap(mListModel->IsListMode());
+}
+
 void lcPartSelectionListView::SetIconSize(int Size)
 {
-	setViewMode(Size ? QListView::IconMode : QListView::ListMode);
 	setIconSize(QSize(Size, Size));
 	lcSetProfileInt(LC_PROFILE_PARTS_LIST_ICONS, Size);
 	mListModel->SetIconSize(Size);
+	UpdateViewMode();
 }
 
 void lcPartSelectionListView::startDrag(Qt::DropActions SupportedActions)
@@ -544,7 +568,7 @@ void lcPartSelectionListView::startDrag(Qt::DropActions SupportedActions)
 }
 
 lcPartSelectionWidget::lcPartSelectionWidget(QWidget* Parent)
-	: QWidget(Parent), mFilterAction(NULL)
+	: QWidget(Parent), mFilterAction(nullptr)
 {
 	mSplitter = new QSplitter(this);
 	mSplitter->setOrientation(Qt::Vertical);
@@ -638,6 +662,11 @@ void lcPartSelectionWidget::SaveState(QSettings& Settings)
 	Settings.setValue("PartSelectionSplitter", QVariant::fromValue(Sizes));
 }
 
+void lcPartSelectionWidget::DisableIconMode()
+{
+	mPartsWidget->SetNoIcons();
+}
+
 void lcPartSelectionWidget::DockLocationChanged(Qt::DockWidgetArea Area)
 {
 	if (Area == Qt::LeftDockWidgetArea || Area == Qt::RightDockWidgetArea)
