@@ -1,5 +1,4 @@
-#ifndef _LC_TEXTURE_H_
-#define _LC_TEXTURE_H_
+#pragma once
 
 #define LC_TEXTURE_WRAPU         0x01
 #define LC_TEXTURE_WRAPV         0x02
@@ -51,6 +50,16 @@ public:
 		return mRefCount;
 	}
 
+	void SetTemporary(bool Temporary)
+	{
+		mTemporary = Temporary;
+	}
+
+	bool IsTemporary() const
+	{
+		return mTemporary;
+	}
+
 	int mWidth;
 	int mHeight;
 	char mName[LC_TEXTURE_NAME_LEN];
@@ -59,6 +68,7 @@ public:
 protected:
 	bool Load();
 
+	bool mTemporary;
 	int mRefCount;
 };
 
@@ -67,4 +77,3 @@ void lcReleaseTexture(lcTexture* Texture);
 
 extern lcTexture* gGridTexture;
 
-#endif // _LC_TEXTURE_H_
