@@ -14,7 +14,7 @@
 lcZipFile::lcZipFile()
 {
 	mModified = false;
-	mFile = NULL;
+	mFile = nullptr;
 }
 
 lcZipFile::~lcZipFile()
@@ -22,15 +22,15 @@ lcZipFile::~lcZipFile()
 	delete mFile;
 }
 
-bool lcZipFile::OpenRead(const char* FilePath)
+bool lcZipFile::OpenRead(const QString& FileName)
 {
-	lcDiskFile* File = new lcDiskFile();
+	lcDiskFile* File = new lcDiskFile(FileName);
 	mFile = File;
 
-	if (!File->Open(FilePath, "rb") || !Open())
+	if (!File->Open(QIODevice::ReadOnly) || !Open())
 	{
 		delete File;
-		mFile = NULL;
+		mFile = nullptr;
 		return false;
 	}
 
@@ -43,41 +43,29 @@ bool lcZipFile::OpenRead(lcFile* File)
 
 	if (!Open())
 	{
-		mFile = NULL;
+		mFile = nullptr;
 		return false;
 	}
 
 	return true;
 }
 
-bool lcZipFile::OpenWrite(const char* FilePath, bool Append)
+bool lcZipFile::OpenWrite(const QString& FileName)
 {
-	lcDiskFile* File = new lcDiskFile();
+	lcDiskFile* File = new lcDiskFile(FileName);
 	mFile = File;
 
-	if (Append)
-	{
-		if (!File->Open(FilePath, "r+b") || !Open())
-		{
-			delete File;
-			mFile = NULL;
-			return false;
-		}
-	}
-	else
-	{
-		mNumEntries = 0;
-		mCentralDirSize = 0;
-		mCentralDirOffset = 0;
-		mBytesBeforeZipFile = 0;
-		mCentralPos = 0;
+	mNumEntries = 0;
+	mCentralDirSize = 0;
+	mCentralDirOffset = 0;
+	mBytesBeforeZipFile = 0;
+	mCentralPos = 0;
 
-		if (!File->Open(FilePath, "wb"))
-		{
-			delete File;
-			mFile = NULL;
-			return false;
-		}
+	if (!File->Open(QIODevice::WriteOnly))
+	{
+		delete File;
+		mFile = nullptr;
+		return false;
 	}
 
 	return true;
@@ -382,7 +370,7 @@ bool lcZipFile::ReadCentralDir()
 		lcZipFileInfo& FileInfo = mFiles.Add();
 		long Seek = 0;
 
-		FileInfo.write_buffer = NULL;
+		FileInfo.write_buffer = nullptr;
 		FileInfo.deleted = false;
 
 		if (mFile->ReadU32(&Magic, 1) != 1 || Magic != 0x02014b50)
@@ -466,7 +454,7 @@ bool lcZipFile::ReadCentralDir()
 		Seek -= SizeRead;
 /*
 		// Read extrafield
-		if ((err==UNZ_OK) && (extraField!=NULL))
+		if ((err==UNZ_OK) && (extraField!=nullptr))
 		{
 			ZPOS64_T uSizeRead ;
 			if (file_info.size_file_extra<extraFieldBufferSize)
@@ -554,7 +542,7 @@ bool lcZipFile::ReadCentralDir()
 			}
 		}
 /*
-		if ((err==UNZ_OK) && (szComment!=NULL))
+		if ((err==UNZ_OK) && (szComment!=nullptr))
 		{
 			uLong uSizeRead ;
 			if (file_info.size_file_comment<commentBufferSize)
@@ -713,7 +701,7 @@ bool lcZipFile::ExtractFile(int FileIndex, lcMemFile& File, lcuint32 MaxLength)
 
 			int err = inflate(&Stream,flush);
 
-			if ((err >= 0) && (Stream.msg != NULL))
+			if ((err >= 0) && (Stream.msg != nullptr))
 				err = Z_DATA_ERROR;
 
 			TotalOutAfter = Stream.total_out;
