@@ -11,6 +11,7 @@
 #include "lc_application.h"
 #include "lc_library.h"
 #include "lc_context.h"
+#include "lc_scene.h"
 #include "lc_qutils.h"
 #include "lc_synth.h"
 
@@ -19,14 +20,14 @@
 lcPiece::lcPiece(PieceInfo* Info)
 	: lcObject(LC_OBJECT_PIECE)
 {
-	mMesh = NULL;
+	mMesh = nullptr;
 	SetPieceInfo(Info, true);
 	mState = 0;
 	mColorIndex = gDefaultColor;
 	mColorCode = 16;
 	mStepShow = 1;
 	mStepHide = LC_STEP_MAX;
-	mGroup = NULL;
+	mGroup = nullptr;
 	mFileLine = -1;
 	mPivotMatrix = lcMatrix44Identity();
 
@@ -34,6 +35,27 @@ lcPiece::lcPiece(PieceInfo* Info)
 	ChangeKey(mRotationKeys, lcMatrix33Identity(), 1, true);
 }
 
+lcPiece::lcPiece(const lcPiece& Other)
+	: lcObject(LC_OBJECT_PIECE)
+{
+	mMesh = nullptr;
+	SetPieceInfo(Other.mPieceInfo, true);
+	mState = 0;
+	mColorIndex = Other.mColorIndex;
+	mColorCode = Other.mColorCode;
+	mStepShow = Other.mStepShow;
+	mStepHide = Other.mStepHide;
+	mGroup = Other.mGroup;
+	mFileLine = -1;
+	mPivotMatrix = Other.mPivotMatrix;
+
+	mPositionKeys = Other.mPositionKeys;
+	mRotationKeys = Other.mRotationKeys;
+	mControlPoints = Other.mControlPoints;
+
+	UpdateMesh();
+}
+
 lcPiece::~lcPiece()
 {
 	if (mPieceInfo)
@@ -55,9 +77,9 @@ void lcPiece::SetPieceInfo(PieceInfo* Info, bool Wait)
 
 	mControlPoints.RemoveAll();
 	delete mMesh;
-	mMesh = NULL;
+	mMesh = nullptr;
 
-	lcSynthInfo* SynthInfo = mPieceInfo ? mPieceInfo->GetSynthInfo() : NULL;
+	lcSynthInfo* SynthInfo = mPieceInfo ? mPieceInfo->GetSynthInfo() : nullptr;
 
 	if (SynthInfo)
 	{
@@ -270,7 +292,7 @@ bool lcPiece::FileLoad(lcFile& file)
   else
 	  file.ReadBuffer(name, LC_PIECE_NAME_LEN);
 
-	PieceInfo* pInfo = lcGetPiecesLibrary()->FindPiece(name, NULL, true, false);
+	PieceInfo* pInfo = lcGetPiecesLibrary()->FindPiece(name, nullptr, true, false);
 	SetPieceInfo(pInfo, true);
 
 	// 11 (0.77)
@@ -459,7 +481,7 @@ void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
 			lcVector3 PointEnd = lcMul31(End, InverseTransform);
 
 			float Distance;
-			if (!lcBoundingBoxRayIntersectDistance(Min, Max, PointStart, PointEnd, &Distance, NULL) || (Distance >= ObjectRayTest.Distance))
+			if (!lcBoundingBoxRayIntersectDistance(Min, Max, PointStart, PointEnd, &Distance, nullptr) || (Distance >= ObjectRayTest.Distance))
 				continue;
 
 			ObjectRayTest.ObjectSection.Object = const_cast<lcPiece*>(this);
@@ -485,7 +507,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 	const lcVector3& Max = BoundingBox.Max;
 	lcVector3 Edge((Max - Min) * 0.33f);
 
-	float Verts[48][3] =
+	float LineVerts[48][3] =
 	{
 		{ Max[0], Max[1], Max[2] }, { Max[0] - Edge[0], Max[1], Max[2] },
 		{ Max[0], Max[1], Max[2] }, { Max[0], Max[1] - Edge[1], Max[2] },
@@ -520,7 +542,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 		{ Min[0], Min[1], Min[2] }, { Min[0], Min[1], Min[2] + Edge[2] },
 	};
 
-	Context->SetProgram(LC_PROGRAM_SIMPLE);
+	Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	Context->SetWorldMatrix(mModelWorld);
 
 	if (IsFocused(LC_PIECE_SECTION_POSITION))
@@ -528,8 +550,8 @@ void lcPiece::DrawInterface(lcContext* Context) const
 	else
 		Context->SetInterfaceColor(LC_COLOR_SELECTED);
 
-	Context->SetVertexBufferPointer(Verts);
-	Context->SetVertexFormat(0, 3, 0, 0);
+	Context->SetVertexBufferPointer(LineVerts);
+	Context->SetVertexFormatPosition(3);
 
 	Context->DrawPrimitives(GL_LINES, 0, 48);
 
@@ -550,7 +572,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 		Context->SetWorldMatrix(lcMul(mPivotMatrix, mModelWorld));
 
 		Context->SetVertexBufferPointer(Verts);
-		Context->SetVertexFormat(0, 3, 0, 0);
+		Context->SetVertexFormatPosition(3);
 		Context->SetIndexBufferPointer(Indices);
 
 		Context->DrawIndexedPrimitives(GL_LINES, 24, GL_UNSIGNED_SHORT, 0);
@@ -579,7 +601,6 @@ void lcPiece::DrawInterface(lcContext* Context) const
 			7, 3, 2, 6, 7, 2, 0, 3, 7, 0, 7, 4, 6, 2, 1, 5, 6, 1
 		};
 
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		glEnable(GL_CULL_FACE);
 
@@ -588,7 +609,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 			Context->SetWorldMatrix(lcMul(mControlPoints[ControlPointIdx].Transform, mModelWorld));
 
 			Context->SetVertexBufferPointer(Verts);
-			Context->SetVertexFormat(0, 3, 0, 0);
+			Context->SetVertexFormatPosition(3);
 			Context->SetIndexBufferPointer(Indices);
 
 			if (IsFocused(LC_PIECE_SECTION_CONTROL_POINT_1 + ControlPointIdx))
@@ -604,7 +625,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 	}
 }
 
-void lcPiece::AddRenderMeshes(lcScene& Scene, bool DrawInterface) const
+void lcPiece::AddRenderMeshes(lcScene& Scene, bool DrawInterface, bool Highlight) const
 {
 	bool Focused, Selected;
 
@@ -620,29 +641,25 @@ void lcPiece::AddRenderMeshes(lcScene& Scene, bool DrawInterface) const
 	}
 
 	if (!mMesh)
-		mPieceInfo->AddRenderMeshes(Scene, mModelWorld, mColorIndex, Focused, Selected);
+		mPieceInfo->AddRenderMeshes(Scene, mModelWorld, mColorIndex, Focused, Selected, Highlight);
 	else
-	{
-		lcRenderMesh RenderMesh;
-
-		RenderMesh.WorldMatrix = mModelWorld;
-		RenderMesh.Mesh = mMesh;
-		RenderMesh.ColorIndex = mColorIndex;
-		RenderMesh.State = Focused ? LC_RENDERMESH_FOCUSED : (Selected ? LC_RENDERMESH_SELECTED : LC_RENDERMESH_NONE);
-		RenderMesh.Distance = fabsf(lcMul31(mModelWorld[3], Scene.mViewMatrix).z);
-		RenderMesh.LodIndex = RenderMesh.Mesh->GetLodIndex(RenderMesh.Distance);
-
-		bool Translucent = lcIsColorTranslucent(mColorIndex);
-
-		if ((mPieceInfo->mFlags & (LC_PIECE_HAS_SOLID | LC_PIECE_HAS_LINES)) || ((mPieceInfo->mFlags & LC_PIECE_HAS_DEFAULT) && !Translucent))
-			Scene.mOpaqueMeshes.Add(RenderMesh);
-
-		if ((mPieceInfo->mFlags & LC_PIECE_HAS_TRANSLUCENT) || ((mPieceInfo->mFlags & LC_PIECE_HAS_DEFAULT) && Translucent))
-			Scene.mTranslucentMeshes.Add(RenderMesh);
-	}
+		Scene.AddMesh(mMesh, mModelWorld, mColorIndex, Focused ? LC_RENDERMESH_FOCUSED : (Selected ? LC_RENDERMESH_SELECTED : LC_RENDERMESH_NONE), mPieceInfo->mFlags);
 
 	if (Selected)
-		Scene.mInterfaceObjects.Add(this);
+		Scene.AddInterfaceObject(this);
+}
+
+void lcPiece::SubModelAddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, bool Focused, bool Selected) const
+{
+	int ColorIndex = mColorIndex;
+
+	if (ColorIndex == gDefaultColor)
+		ColorIndex = DefaultColorIndex;
+
+	if (!mMesh)
+		mPieceInfo->AddRenderMeshes(Scene, lcMul(mModelWorld, WorldMatrix), ColorIndex, Focused, Selected, false);
+	else
+		Scene.AddMesh(mMesh, lcMul(mModelWorld, WorldMatrix), ColorIndex, Focused ? LC_RENDERMESH_FOCUSED : (Selected ? LC_RENDERMESH_SELECTED : LC_RENDERMESH_NONE), mPieceInfo->mFlags);
 }
 
 void lcPiece::Move(lcStep Step, bool AddKey, const lcVector3& Distance)
@@ -744,7 +761,11 @@ lcuint32 lcPiece::GetAllowedTransforms() const
 	if (Section == LC_PIECE_SECTION_POSITION || Section == LC_PIECE_SECTION_INVALID)
 		return LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
 
-	if (mPieceInfo->GetSynthInfo()->IsCurve())
+	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
+	if (!SynthInfo)
+		return 0;
+
+	if (SynthInfo->IsCurve())
 		return LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z | LC_OBJECT_TRANSFORM_SCALE_X;
 	else
 		return LC_OBJECT_TRANSFORM_MOVE_Z;
@@ -829,7 +850,7 @@ void lcPiece::CompareBoundingBox(lcVector3& Min, lcVector3& Max) const
 
 lcGroup* lcPiece::GetTopGroup()
 {
-	return mGroup ? mGroup->GetTopGroup() : NULL;
+	return mGroup ? mGroup->GetTopGroup() : nullptr;
 }
 
 void lcPiece::UpdatePosition(lcStep Step)
@@ -844,5 +865,5 @@ void lcPiece::UpdateMesh()
 {
 	delete mMesh;
 	lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
-	mMesh = SynthInfo ? SynthInfo->CreateMesh(mControlPoints) : NULL;
+	mMesh = SynthInfo ? SynthInfo->CreateMesh(mControlPoints) : nullptr;
 }
