@@ -12,12 +12,12 @@ public:
 		return mTexture != 0;
 	}
 
-	void MakeCurrent()
+	GLuint GetTexture() const
 	{
-		glBindTexture(GL_TEXTURE_2D, mTexture);
+		return mTexture;
 	}
 
-	bool Load();
+	bool Load(lcContext* Context);
 	void Release();
 
 	void PrintText(lcContext* Context, float Left, float Top, float Z, const char* Text) const;
