@@ -3,7 +3,6 @@
 #include "lc_mainwindow.h"
 #include "camera.h"
 #include "view.h"
-#include "system.h"
 #include "tr.h"
 #include "texfont.h"
 #include "lc_texture.h"
@@ -17,7 +16,8 @@ lcIndexBuffer View::mRotateMoveIndexBuffer;
 View::View(lcModel* Model)
 {
 	mModel = Model;
-	mCamera = NULL;
+	mCamera = nullptr;
+	mHighlight = false;
 	memset(mGridSettings, 0, sizeof(mGridSettings));
 
 	mDragState = LC_DRAGSTATE_NONE;
@@ -199,7 +199,7 @@ void View::CreateSelectMoveOverlayMesh(lcContext* Context)
 void View::DestroyResources(lcContext* Context)
 {
 	delete gGridTexture;
-	gGridTexture = NULL;
+	gGridTexture = nullptr;
 
 	Context->DestroyVertexBuffer(mRotateMoveVertexBuffer);
 	Context->DestroyIndexBuffer(mRotateMoveIndexBuffer);
@@ -240,6 +240,20 @@ void View::SetCamera(lcCamera* Camera, bool ForceCopy)
 	}
 }
 
+void View::SetCamera(const char* CameraName)
+{
+	const lcArray<lcCamera*>& Cameras = mModel->GetCameras();
+
+	for (int CameraIdx = 0; CameraIdx < Cameras.GetSize(); CameraIdx++)
+	{
+		if (qstricmp(CameraName, Cameras[CameraIdx]->m_strName) == 0)
+		{
+			SetCameraIndex(CameraIdx);
+			return;
+		}
+	}
+}
+
 void View::SetCameraIndex(int Index)
 {
 	const lcArray<lcCamera*>& Cameras = mModel->GetCameras();
@@ -341,6 +355,7 @@ void View::ShowContextMenu() const
 	Popup->addAction(Actions[LC_EDIT_CUT]);
 	Popup->addAction(Actions[LC_EDIT_COPY]);
 	Popup->addAction(Actions[LC_EDIT_PASTE]);
+	Popup->addAction(Actions[LC_PIECE_DUPLICATE]);
 
 	Popup->addSeparator();
 
@@ -477,7 +492,7 @@ lcObjectSection View::FindObjectUnderPointer(bool PiecesOnly) const
 	ObjectRayTest.Start = StartEnd[0];
 	ObjectRayTest.End = StartEnd[1];
 	ObjectRayTest.Distance = FLT_MAX;
-	ObjectRayTest.ObjectSection.Object = NULL;
+	ObjectRayTest.ObjectSection.Object = nullptr;
 	ObjectRayTest.ObjectSection.Section = 0;;
 
 	mModel->RayTest(ObjectRayTest);
@@ -547,16 +562,16 @@ lcArray<lcObject*> View::FindObjectsInBox(float x1, float y1, float x2, float y2
 
 void View::OnDraw()
 {
-	bool DrawInterface = mWidget != NULL;
+	bool DrawInterface = mWidget != nullptr;
 
-	mModel->GetScene(mScene, mCamera, DrawInterface);
+	mModel->GetScene(mScene, mCamera, DrawInterface, mHighlight);
 
 	if (DrawInterface && mTrackTool == LC_TRACKTOOL_INSERT)
 	{
 		PieceInfo* Info = gMainWindow->GetCurrentPieceInfo();
 
 		if (Info)
-			Info->AddRenderMeshes(mScene, GetPieceInsertPosition(), gMainWindow->mColorIndex, true, true);
+			Info->AddRenderMeshes(mScene, GetPieceInsertPosition(), gMainWindow->mColorIndex, true, true, false);
 	}
 
 	mContext->SetDefaultState();
@@ -566,74 +581,19 @@ void View::OnDraw()
 
 	const lcPreferences& Preferences = lcGetPreferences();
 
-	mContext->SetViewMatrix(mCamera->mWorldView);
 	mContext->SetProjectionMatrix(GetProjectionMatrix());
-	mContext->SetProgram(LC_PROGRAM_SIMPLE); // todo: lighting
 
-#ifndef LC_OPENGLES
-	const lcModelProperties& Properties = mModel->GetProperties();
-	if (Preferences.mLightingMode != LC_LIGHTING_FLAT)
-	{
-		glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT);
-		glEnable(GL_COLOR_MATERIAL);
-		glShadeModel(GL_SMOOTH);
-
-		float Shininess = 64.0f;
-		glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &Shininess);
-		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, lcVector4(0.8f, 0.8f, 0.8f, 1.0f));
-		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, lcVector4(0.8f, 0.8f, 0.8f, 1.0f));
-
-		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lcVector4(Properties.mAmbientColor, 1.0f));
-
-//		for (int LightIdx = 0; LightIdx < mLights.GetSize(); LightIdx++)
-//			mLights[LightIdx]->Setup(LightIdx);
-
-		glEnable(GL_LIGHTING);
-	}
-	else
-	{
-		glDisable(GL_LIGHTING);
-		glDisable(GL_COLOR_MATERIAL);
-		glShadeModel(GL_FLAT);
-	}
-
-	if (Properties.mFogEnabled)
-	{
-		glFogi(GL_FOG_MODE, GL_EXP);
-		glFogf(GL_FOG_DENSITY, Properties.mFogDensity);
-		glFogfv(GL_FOG_COLOR, lcVector4(Properties.mFogColor, 1.0f));
-		glEnable(GL_FOG);
-	}
-#endif
-	
 	mContext->SetLineWidth(Preferences.mLineWidth);
 
-	mContext->DrawOpaqueMeshes(mScene.mOpaqueMeshes);
-	mContext->DrawTranslucentMeshes(mScene.mTranslucentMeshes);
-
-	mContext->UnbindMesh(); // context remove
-
-#ifndef LC_OPENGLES
-	if (Preferences.mLightingMode != LC_LIGHTING_FLAT)
-	{
-		glDisable(GL_LIGHTING);
-		glDisable(GL_COLOR_MATERIAL);
-		glShadeModel(GL_FLAT);
-	}
-
-	if (Properties.mFogEnabled)
-		glDisable(GL_FOG);
-#endif
+	mScene.Draw(mContext);
 
 	if (DrawInterface)
 	{
-		mContext->SetProgram(LC_PROGRAM_SIMPLE);
-		mContext->DrawInterfaceObjects(mScene.mInterfaceObjects);
+		mScene.DrawInterfaceObjects(mContext);
 
 		mContext->SetLineWidth(1.0f);
 
-		if (Preferences.mDrawGridStuds || Preferences.mDrawGridLines)
-			DrawGrid();
+		DrawGrid();
 
 		if (Preferences.mDrawAxes)
 			DrawAxes();
@@ -653,11 +613,13 @@ void View::OnDraw()
 
 		DrawViewport();
 	}
+
+	mContext->ClearResources();
 }
 
 void View::DrawSelectMoveOverlay()
 {
-	mContext->SetProgram(LC_PROGRAM_SIMPLE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	mContext->SetViewMatrix(mCamera->mWorldView);
 	mContext->SetProjectionMatrix(GetProjectionMatrix());
 
@@ -677,7 +639,7 @@ void View::DrawSelectMoveOverlay()
 
 	mContext->SetIndexBuffer(mRotateMoveIndexBuffer);
 	mContext->SetVertexBuffer(mRotateMoveVertexBuffer);
-	mContext->SetVertexFormat(0, 3, 0, 0);
+	mContext->SetVertexFormatPosition(3);
 
 	lcObject* Focus = mModel->GetFocusObject();
 	lcuint32 AllowedTransforms = Focus ? Focus->GetAllowedTransforms() : LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
@@ -762,7 +724,6 @@ void View::DrawSelectMoveOverlay()
 
 	if ((mTrackTool == LC_TRACKTOOL_MOVE_XY) || (mTrackTool == LC_TRACKTOOL_MOVE_XZ) || (mTrackTool == LC_TRACKTOOL_MOVE_YZ))
 	{
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 
 		mContext->SetColor(0.8f, 0.8f, 0.0f, 0.3f);
@@ -782,7 +743,7 @@ void View::DrawSelectMoveOverlay()
 		lcPiece* Piece = (lcPiece*)Focus;
 		lcuint32 Section = Piece->GetFocusSection();
 
-		if (Section >= LC_PIECE_SECTION_CONTROL_POINT_1 && Section <= LC_PIECE_SECTION_CONTROL_POINT_8 && Piece->mPieceInfo->GetSynthInfo()->IsCurve())
+		if (Section >= LC_PIECE_SECTION_CONTROL_POINT_1 && Section <= LC_PIECE_SECTION_CONTROL_POINT_8 && Piece->mPieceInfo->GetSynthInfo() && Piece->mPieceInfo->GetSynthInfo()->IsCurve())
 		{
 			int ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_1;
 			float Strength = Piece->GetControlPoints()[ControlPointIndex].Scale;
@@ -830,7 +791,7 @@ void View::DrawSelectMoveOverlay()
 
 			mContext->SetVertexBufferPointer(Verts);
 			mContext->ClearIndexBuffer();
-			mContext->SetVertexFormat(0, 3, 0, 0);
+			mContext->SetVertexFormatPosition(3);
 
 			mContext->DrawPrimitives(GL_LINES, 0, 2);
 			mContext->DrawPrimitives(GL_TRIANGLE_STRIP, 2, 18);
@@ -839,8 +800,6 @@ void View::DrawSelectMoveOverlay()
 	}
 
 	glEnable(GL_DEPTH_TEST);
-
-	mContext->ClearIndexBuffer(); // context remove
 }
 
 void View::DrawRotateOverlay()
@@ -848,7 +807,7 @@ void View::DrawRotateOverlay()
 	const float OverlayScale = GetOverlayScale();
 	const float OverlayRotateRadius = 2.0f;
 
-	mContext->SetProgram(LC_PROGRAM_SIMPLE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	mContext->SetViewMatrix(mCamera->mWorldView);
 	mContext->SetProjectionMatrix(GetProjectionMatrix());
 
@@ -910,7 +869,6 @@ void View::DrawRotateOverlay()
 
 			mContext->SetWorldMatrix(WorldMatrix);
 
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 
 			lcVector3 Verts[33];
@@ -918,7 +876,7 @@ void View::DrawRotateOverlay()
 			int NumVerts = 1;
 
 			mContext->SetVertexBufferPointer(Verts);
-			mContext->SetVertexFormat(0, 3, 0, 0);
+			mContext->SetVertexFormatPosition(3);
 
 			float StartAngle;
 			int i = 0;
@@ -980,7 +938,7 @@ void View::DrawRotateOverlay()
 		mContext->SetWorldMatrix(lcMatrix44Identity());
 
 		mContext->SetVertexBufferPointer(Verts);
-		mContext->SetVertexFormat(0, 3, 0, 0);
+		mContext->SetVertexFormatPosition(3);
 
 		mContext->DrawPrimitives(GL_LINE_LOOP, 0, 32);
 	}
@@ -991,8 +949,7 @@ void View::DrawRotateOverlay()
 	// Transform ViewDir to local space.
 	ViewDir = lcMul(ViewDir, lcMatrix33AffineInverse(RelativeRotation));
 
-	lcMatrix44 WorldMatrix = lcMatrix44(RelativeRotation, OverlayCenter);
-	mContext->SetWorldMatrix(WorldMatrix);
+	mContext->SetWorldMatrix(lcMatrix44(RelativeRotation, OverlayCenter));
 
 	// Draw each axis circle.
 	for (int i = 0; i < 3; i++)
@@ -1053,7 +1010,7 @@ void View::DrawRotateOverlay()
 		}
 
 		mContext->SetVertexBufferPointer(Verts);
-		mContext->SetVertexFormat(0, 3, 0, 0);
+		mContext->SetVertexFormatPosition(3);
 
 		mContext->DrawPrimitives(GL_LINES, 0, NumVerts);
 	}
@@ -1111,7 +1068,7 @@ void View::DrawRotateOverlay()
 			Verts[5] = lcVector3(0.0f, StartY - OverlayScale * OverlayRotateArrowCapSize, EndZ + TipZ);
 
 			mContext->SetVertexBufferPointer(Verts);
-			mContext->SetVertexFormat(0, 3, 0, 0);
+			mContext->SetVertexFormatPosition(3);
 
 			mContext->DrawPrimitives(GL_LINES, 0, 6);
 		}
@@ -1119,15 +1076,11 @@ void View::DrawRotateOverlay()
 		// Draw text.
 		lcVector3 ScreenPos = ProjectPoint(OverlayCenter);
 
-		mContext->SetProgram(LC_PROGRAM_TEXTURE);
+		mContext->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_MODULATE);
 		mContext->SetWorldMatrix(lcMatrix44Identity());
 		mContext->SetViewMatrix(lcMatrix44Translation(lcVector3(0.375, 0.375, 0.0)));
 		mContext->SetProjectionMatrix(lcMatrix44Ortho(0.0f, mWidth, 0.0f, mHeight, -1.0f, 1.0f));
-
-		mContext->SetTextureMode(LC_TEXTURE_MODULATE);
-		gTexFont.MakeCurrent();
-		glEnable(GL_TEXTURE_2D);
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		mContext->BindTexture(gTexFont.GetTexture());
 		glEnable(GL_BLEND);
 
 		char buf[32];
@@ -1140,7 +1093,6 @@ void View::DrawRotateOverlay()
 		gTexFont.PrintText(mContext, ScreenPos[0] - (cx / 2), ScreenPos[1] + (cy / 2), 0.0f, buf);
 
 		glDisable(GL_BLEND);
-		glDisable(GL_TEXTURE_2D);
 	}
 
 	glEnable(GL_DEPTH_TEST);
@@ -1148,7 +1100,7 @@ void View::DrawRotateOverlay()
 
 void View::DrawSelectZoomRegionOverlay()
 {
-	mContext->SetProgram(LC_PROGRAM_SIMPLE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	mContext->SetWorldMatrix(lcMatrix44Identity());
 	mContext->SetViewMatrix(lcMatrix44Translation(lcVector3(0.375, 0.375, 0.0)));
 	mContext->SetProjectionMatrix(lcMatrix44Ortho(0.0f, mWidth, 0.0f, mHeight, -1.0f, 1.0f));
@@ -1210,11 +1162,10 @@ void View::DrawSelectZoomRegionOverlay()
 		{ Right - BorderX, Top - BorderY },
 	};
 
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_BLEND);
 
 	mContext->SetVertexBufferPointer(Verts);
-	mContext->SetVertexFormat(0, 2, 0, 0);
+	mContext->SetVertexFormatPosition(2);
 
 	mContext->SetColor(0.25f, 0.25f, 1.0f, 1.0f);
 	mContext->DrawPrimitives(GL_TRIANGLE_STRIP, 0, 10);
@@ -1224,8 +1175,6 @@ void View::DrawSelectZoomRegionOverlay()
 
 	glDisable(GL_BLEND);
 	glEnable(GL_DEPTH_TEST);
-
-	mContext->ClearVertexBuffer(); // context remove
 }
 
 void View::DrawRotateViewOverlay()
@@ -1237,7 +1186,7 @@ void View::DrawRotateViewOverlay()
 	w = mWidth;
 	h = mHeight;
 
-	mContext->SetProgram(LC_PROGRAM_SIMPLE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	mContext->SetWorldMatrix(lcMatrix44Identity());
 	mContext->SetViewMatrix(lcMatrix44Translation(lcVector3(0.375, 0.375, 0.0)));
 	mContext->SetProjectionMatrix(lcMatrix44Ortho(0, w, 0, h, -1, 1));
@@ -1278,7 +1227,7 @@ void View::DrawRotateViewOverlay()
 	*CurVert++ = cx - r + OverlayCameraSquareSize; *CurVert++ = cy - OverlayCameraSquareSize;
 
 	mContext->SetVertexBufferPointer(Verts);
-	mContext->SetVertexFormat(0, 2, 0, 0);
+	mContext->SetVertexFormatPosition(2);
 
 	GLushort Indices[64 + 32] = 
 	{
@@ -1292,13 +1241,13 @@ void View::DrawRotateViewOverlay()
 	mContext->DrawIndexedPrimitives(GL_LINES, 96, GL_UNSIGNED_SHORT, 0);
 
 	glEnable(GL_DEPTH_TEST);
-
-	mContext->ClearVertexBuffer(); // context remove
 }
 
 void View::DrawGrid()
 {
 	const lcPreferences& Preferences = lcGetPreferences();
+	if (!Preferences.mDrawGridStuds && !Preferences.mDrawGridLines)
+		return;
 
 	mContext->SetWorldMatrix(lcMatrix44Identity());
 
@@ -1363,6 +1312,8 @@ void View::DrawGrid()
 			VertexBufferSize += 2 * (MaxX - MinX + MaxY - MinY + 2) * 3 * sizeof(float);
 
 		float* Verts = (float*)malloc(VertexBufferSize);
+		if (!Verts)
+			return;
 		float* CurVert = Verts;
 
 		if (Preferences.mDrawGridStuds)
@@ -1443,19 +1394,15 @@ void View::DrawGrid()
 
 	if (Preferences.mDrawGridStuds)
 	{
-		mContext->SetTextureMode(LC_TEXTURE_MODULATE);
-		glBindTexture(GL_TEXTURE_2D, gGridTexture->mTexture);
-		glEnable(GL_TEXTURE_2D);
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		mContext->BindTexture(gGridTexture->mTexture);
 		glEnable(GL_BLEND);
 
-		mContext->SetProgram(LC_PROGRAM_TEXTURE);
+		mContext->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_MODULATE);
 		mContext->SetColor(lcVector4FromColor(Preferences.mGridStudColor));
 
-		mContext->SetVertexFormat(0, 3, 2, 0);
+		mContext->SetVertexFormat(0, 3, 0, 2, 0, false);
 		mContext->DrawPrimitives(GL_TRIANGLE_STRIP, 0, 4);
 
-		glDisable(GL_TEXTURE_2D);
 		glDisable(GL_BLEND);
 
 		BufferOffset = 4 * 5 * sizeof(float);
@@ -1464,16 +1411,14 @@ void View::DrawGrid()
 	if (Preferences.mDrawGridLines)
 	{
 		mContext->SetLineWidth(1.0f);
-		mContext->SetProgram(LC_PROGRAM_SIMPLE);
+		mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 		mContext->SetColor(lcVector4FromColor(Preferences.mGridLineColor));
 
 		int NumVerts = 2 * (MaxX - MinX + MaxY - MinY + 2);
 
-		mContext->SetVertexFormat(BufferOffset, 3, 0, 0);
+		mContext->SetVertexFormat(BufferOffset, 3, 0, 0, 0, false);
 		mContext->DrawPrimitives(GL_LINES, 0, NumVerts);
 	}
-
-	mContext->ClearVertexBuffer(); // context remove
 }
 
 void View::DrawAxes()
@@ -1503,13 +1448,13 @@ void View::DrawAxes()
 	lcMatrix44 WorldViewMatrix = mCamera->mWorldView;
 	WorldViewMatrix.SetTranslation(lcVector3(0, 0, 0));
 
-	mContext->SetProgram(LC_PROGRAM_SIMPLE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 	mContext->SetWorldMatrix(lcMatrix44Identity());
 	mContext->SetViewMatrix(lcMul(WorldViewMatrix, TranslationMatrix));
 	mContext->SetProjectionMatrix(lcMatrix44Ortho(0, mWidth, 0, mHeight, -50, 50));
 
 	mContext->SetVertexBufferPointer(Verts);
-	mContext->SetVertexFormat(0, 3, 0, 0);
+	mContext->SetVertexFormatPosition(3);
 	mContext->SetIndexBufferPointer(Indices);
 
 	mContext->SetColor(0.0f, 0.0f, 0.0f, 1.0f);
@@ -1522,12 +1467,9 @@ void View::DrawAxes()
 	mContext->SetColor(0.0f, 0.0f, 0.8f, 1.0f);
 	mContext->DrawIndexedPrimitives(GL_TRIANGLES, 24, GL_UNSIGNED_SHORT, (6 + 48) * 2);
 
-	mContext->SetProgram(LC_PROGRAM_TEXTURE);
+	mContext->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_MODULATE);
 	mContext->SetViewMatrix(TranslationMatrix);
-	mContext->SetTextureMode(LC_TEXTURE_MODULATE);
-	gTexFont.MakeCurrent();
-	glEnable(GL_TEXTURE_2D);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	mContext->BindTexture(gTexFont.GetTexture());
 	glEnable(GL_BLEND);
 
 	float TextBuffer[6 * 5 * 3];
@@ -1539,15 +1481,12 @@ void View::DrawAxes()
 	gTexFont.GetGlyphTriangles(PosZ.x, PosZ.y, PosZ.z, 'Z', TextBuffer + 5 * 6 * 2);
 
 	mContext->SetVertexBufferPointer(TextBuffer);
-	mContext->SetVertexFormat(0, 3, 2, 0);
+	mContext->SetVertexFormat(0, 3, 0, 2, 0, false);
 
 	mContext->SetColor(0.0f, 0.0f, 0.0f, 1.0f);
 	mContext->DrawPrimitives(GL_TRIANGLES, 0, 6 * 3);
 
 	glDisable(GL_BLEND);
-	glDisable(GL_TEXTURE_2D);
-
-	mContext->ClearVertexBuffer(); // context remove
 }
 
 void View::DrawViewport()
@@ -1558,40 +1497,31 @@ void View::DrawViewport()
 
 	glDepthMask(GL_FALSE);
 	glDisable(GL_DEPTH_TEST);
-#ifndef LC_OPENGLES
-	glDisable(GL_LIGHTING);
-#endif
 
 	if (gMainWindow->GetActiveView() == this)
 	{
-		mContext->SetProgram(LC_PROGRAM_SIMPLE);
+		mContext->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 		mContext->SetColor(1.0f, 0.0f, 0.0f, 1.0f);
 		float Verts[8] = { 0.0f, 0.0f, mWidth - 1.0f, 0.0f, mWidth - 1.0f, mHeight - 1.0f, 0.0f, mHeight - 1.0f };
 
 		mContext->SetVertexBufferPointer(Verts);
-		mContext->SetVertexFormat(0, 2, 0, 0);
+		mContext->SetVertexFormatPosition(2);
 		mContext->DrawPrimitives(GL_LINE_LOOP, 0, 4);
 	}
 
-	mContext->ClearVertexBuffer(); // context remove
-
 	const char* CameraName = mCamera->GetName();
 
 	if (CameraName[0])
 	{
-		mContext->SetProgram(LC_PROGRAM_TEXTURE);
+		mContext->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_MODULATE);
 		mContext->SetColor(0.0f, 0.0f, 0.0f, 1.0f);
+		mContext->BindTexture(gTexFont.GetTexture());
 
-		glEnable(GL_TEXTURE_2D);
-		mContext->SetTextureMode(LC_TEXTURE_MODULATE);
-		gTexFont.MakeCurrent();
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 
 		gTexFont.PrintText(mContext, 3.0f, (float)mHeight - 1.0f - 6.0f, 0.0f, CameraName);
 
 		glDisable(GL_BLEND);
-		glDisable(GL_TEXTURE_2D);
 	}
 
 	glDepthMask(GL_TRUE);
@@ -1864,7 +1794,7 @@ void View::UpdateTrackTool()
 
 				lcPiece* Piece = (lcPiece*)Focus;
 
-				if (ControlPointIndex != -1 && Piece->mPieceInfo->GetSynthInfo()->IsCurve())
+				if (ControlPointIndex != -1 && Piece->mPieceInfo->GetSynthInfo() && Piece->mPieceInfo->GetSynthInfo()->IsCurve())
 				{
 					float Strength = Piece->GetControlPoints()[ControlPointIndex].Scale;
 					const float ScaleStart = (2.0f - OverlayScaleRadius) * OverlayScale + Strength;
@@ -2213,8 +2143,6 @@ bool View::IsTrackToolAllowed(lcTrackTool TrackTool, lcuint32 AllowedTransforms)
 
 void View::StartTracking(lcTrackButton TrackButton)
 {
-	LC_ASSERT(mTrackButton == LC_TRACKBUTTON_NONE);
-
 	mTrackButton = TrackButton;
 	mTrackUpdated = false;
 	mMouseDownX = mInputState.x;
@@ -2308,7 +2236,7 @@ void View::StopTracking(bool Accept)
 			if (mInputState.Modifiers & Qt::ControlModifier)
 				mModel->AddToSelection(Objects);
 			else
-				mModel->SetSelectionAndFocus(Objects, NULL, 0);
+				mModel->SetSelectionAndFocus(Objects, nullptr, 0);
 		}
 		break;
 
@@ -2656,10 +2584,10 @@ void View::OnMouseMove()
 				Direction = lcMul(Direction, RelativeRotation);
 
 				lcVector3 Intersection;
-				lcClosestPointsBetweenLines(Center, Center + Direction, CurrentStart, CurrentEnd, &Intersection, NULL);
+				lcClosestPointsBetweenLines(Center, Center + Direction, CurrentStart, CurrentEnd, &Intersection, nullptr);
 
 				lcVector3 MoveStart;
-				lcClosestPointsBetweenLines(Center, Center + Direction, MouseDownStart, MouseDownEnd, &MoveStart, NULL);
+				lcClosestPointsBetweenLines(Center, Center + Direction, MouseDownStart, MouseDownEnd, &MoveStart, nullptr);
 
 				lcVector3 Distance = Intersection - MoveStart;
 				Distance = lcMul(Distance, lcMatrix33AffineInverse(RelativeRotation));
@@ -2703,7 +2631,7 @@ void View::OnMouseMove()
 				Direction = lcMul(Direction, RelativeRotation);
 
 				lcVector3 Intersection;
-				lcClosestPointsBetweenLines(Center, Center + Direction, CurrentStart, CurrentEnd, &Intersection, NULL);
+				lcClosestPointsBetweenLines(Center, Center + Direction, CurrentStart, CurrentEnd, &Intersection, nullptr);
 
 				lcObject* Focus = mModel->GetFocusObject();
 				if (Focus && Focus->IsPiece())
