@@ -7,6 +7,7 @@
 #include "lc_library.h"
 #include "lc_application.h"
 #include "lc_qutils.h"
+#include "lc_glextensions.h"
 #include "pieceinf.h"
 
 lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
@@ -46,7 +47,8 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 	ui->gridLines->setChecked(options->Preferences.mDrawGridLines);
 	ui->gridLineSpacing->setText(QString::number(options->Preferences.mGridLineSpacing));
 	ui->axisIcon->setChecked(options->Preferences.mDrawAxes);
-	ui->enableLighting->setChecked(options->Preferences.mLightingMode != LC_LIGHTING_FLAT);
+	ui->enableLighting->setChecked(options->Preferences.mLightingMode != LC_LIGHTING_UNLIT && gSupportsShaderObjects);
+	ui->enableLighting->setEnabled(gSupportsShaderObjects);
 
 	QPixmap pix(12, 12);
 
@@ -66,11 +68,11 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 
 	updateCommandList();
 	new lcQTreeWidgetColumnStretcher(ui->commandList, 0);
-	commandChanged(NULL);
+	commandChanged(nullptr);
 
 	UpdateMouseTree();
 	new lcQTreeWidgetColumnStretcher(ui->mouseTree, 0);
-	MouseTreeItemChanged(NULL);
+	MouseTreeItemChanged(nullptr);
 }
 
 lcQPreferencesDialog::~lcQPreferencesDialog()
@@ -112,7 +114,7 @@ void lcQPreferencesDialog::accept()
 	options->Preferences.mGridLineSpacing = gridLineSpacing;
 
 	options->Preferences.mDrawAxes = ui->axisIcon->isChecked();
-	options->Preferences.mLightingMode = ui->enableLighting->isChecked() ? LC_LIGHTING_FULL : LC_LIGHTING_FLAT;
+	options->Preferences.mLightingMode = ui->enableLighting->isChecked() ? LC_LIGHTING_FAKE : LC_LIGHTING_UNLIT;
 
 	QDialog::accept();
 }
@@ -151,7 +153,7 @@ void lcQPreferencesDialog::colorClicked()
 {
 	QObject *button = sender();
 	QString title;
-	lcuint32 *color = NULL;
+	lcuint32 *color = nullptr;
 	QColorDialog::ColorDialogOptions dialogOptions;
 
 	if (button == ui->gridStudColor)
@@ -166,6 +168,8 @@ void lcQPreferencesDialog::colorClicked()
 		title = tr("Select Grid Line Color");
 		dialogOptions = 0;
 	}
+	else
+		return;
 
 	QColor oldColor = QColor(LC_RGBA_RED(*color), LC_RGBA_GREEN(*color), LC_RGBA_BLUE(*color), LC_RGBA_ALPHA(*color));
 	QColor newColor = QColorDialog::getColor(oldColor, this, title, dialogOptions);
@@ -421,7 +425,7 @@ bool lcQPreferencesDialog::eventFilter(QObject *object, QEvent *event)
 		return true;
 	}
 
-	return false;
+	return QDialog::eventFilter(object, event);
 }
 
 void lcQPreferencesDialog::updateCommandList()
